/*! Tripetto Collector 0.4.23 - Copyright (C) 2018 Tripetto B.V. - All Rights Reserved */

declare module "@marblecore/callback" {
    /** Explicit exports */
    export { Await } from "@marblecore/callback/lib/await";
    export {
        Callback,
        ICallbackProperties,
        ICallbackPromise
    } from "@marblecore/callback/lib/callback";
    export {
        IAwaitProperties,
        IAwaitPromise
    } from "@marblecore/callback/lib/await";
    /** Default export */
    export { Callback as default } from "@marblecore/callback/lib/callback";
}

declare module "@marblecore/callback/lib/await" {
    /** Dependencies */
    import {
        Callback,
        ICallbackPromise,
        TPromise
    } from "@marblecore/callback/lib/callback";
    /**
     * Type alias for the await function.
     * @param TAwait.pAwait Reference to the await.
     */
    export type TAwait = (pAwait: Await) => void | Await;
    /** Defines the promise interface. */
    export interface IAwaitPromise extends ICallbackPromise<void> {}
    /** Defines the callback properties interface. */
    export interface IAwaitProperties {
        /** Specifies the await function. */
        readonly Await: TAwait;
        /** Specifies the promise function. */
        readonly Promise: TPromise<void>;
        /**
         * Specifies if the callback is conditional. In this case the callback
         * condition must be set to true before a callback is accepted. The callback
         * condition can only be used for asynchronous callbacks.
         */
        readonly Conditional?: boolean;
        /**
         * Specifies if the callback should be continuous. The callback function
         * `Done()` can be called multiple times.
         */
        readonly Continuous?: boolean;
    }
    export class Await extends Callback<void> {
        /**
         * Await for.
         * @param pProperties Specifies the await properties.
         * @return Returns a reference to the await.
         */
        static For(pProperties: IAwaitProperties): Await;
        /**
         * Creates a new await.
         * @param pProperties Specifies the await properties.
         */
        constructor(pProperties: IAwaitProperties);
        /**
         * Terminates the await.
         * @return Returns `true` if the await loop is terminated.
         */
        Done(): boolean;
    }
}

declare module "@marblecore/callback/lib/callback" {
    /**
     * Type alias for the await function.
     * @param TAwait.pCallback Reference to the callback.
     */
    export type TAwait<T> = (pCallback: Callback<T>) => T | Callback<T>;
    /**
     * Type alias for the promise function.
     * @param TPromise.pPromise Contains the promise details.
     */
    export type TPromise<T> =
        | ((pPromise: ICallbackPromise<T>) => void)
        | undefined;
    /** Defines the promise interface. */
    export interface ICallbackPromise<T> {
        /** Specifies if the await was asynchronous. */
        readonly IsAsynchronous: boolean;
        /** Specifies if the await was canceled. */
        readonly IsCanceled: boolean;
        /** Specifies the payload which is returned. */
        readonly Payload: T | undefined;
    }
    /** Defines the callback properties interface. */
    export interface ICallbackProperties<T> {
        /** Specifies the await function. */
        readonly Await: TAwait<T>;
        /** Specifies the promise function. */
        readonly Promise: TPromise<T>;
        /**
         * Specifies if the callback is conditional. In this case the callback
         * condition must be set to true before a callback is accepted. The callback
         * condition can only be used for asynchronous callbacks.
         */
        readonly Conditional?: boolean;
        /**
         * Specifies if the callback should be continuous. The callback property
         * `Payload` or function `Done()` can be called multiple times.
         */
        readonly Continuous?: boolean;
    }
    export class Callback<T> {
        /**
         * Callback template of specified type.
         * @param pProperties Specifies the callback properties.
         * @return Returns a reference to the callback.
         */
        static Of<Type>(pProperties: ICallbackProperties<Type>): Callback<Type>;
        /**
         * Creates a new callback.
         * @param pProperties Specifies the callback properties.
         */
        constructor(pProperties: ICallbackProperties<T>);
        /** Sets the callback condition state. */
        Condition: boolean;
        /** Verifies if the callback pointer is valid. */
        readonly IsValid: boolean;
        /** Sets the payload of the callback function. */
        Payload: T | undefined;
        /** Retrieves the promise invoke count. */
        readonly Promises: number;
        /**
         * Terminates the callback.
         * @return Returns `true` if the callback loop is terminated.
         */
        Done(): boolean;
        /**
         * Cancels the callback loop.
         * @return Returns `true` if the callback loop is terminated.
         */
        Cancel(): boolean;
    }
}
