/*! Tripetto Collector 0.4.23 - Copyright (C) 2018 Tripetto B.V. - All Rights Reserved */

declare module "@marblecore/serializer" {
    /** Explicit exports */
    export {
        ISerialize,
        TSerializeTypes
    } from "@marblecore/serializer/lib/interface";
    export { Serialize } from "@marblecore/serializer/lib/serialize";
    export { Deserialize } from "@marblecore/serializer/lib/deserialize";
    /** Decorators */
    export { deserialize } from "@marblecore/serializer/decorators/deserialize";
    export {
        deserializer
    } from "@marblecore/serializer/decorators/deserializer";
    export { serializer } from "@marblecore/serializer/decorators/serializer";
    export { serialize } from "@marblecore/serializer/decorators/serialize";
}

declare module "@marblecore/serializer/lib/interface" {
    export type TSerializePrimitives =
        | string
        | number
        | boolean
        | undefined
        | null
        | ISerialize;
    export type TSerializeTypes = TSerializePrimitives | TSerializePrimitives[];
    export interface ISerialize {
        /** Retrieves the specified property. */
        readonly [sProperty: string]: TSerializeTypes;
    }
}

declare module "@marblecore/serializer/lib/serialize" {
    import { ISerialize } from "@marblecore/serializer/lib/interface";
    /**
     * Serializes a source.
     * @param pSource Reference to the source.
     * @param pData Reference to the data object which receives the serialized data.
     */
    export function Serialize<T, S extends ISerialize>(pSource: T, pData: S): S;
}

declare module "@marblecore/serializer/lib/deserialize" {
    import { ISerialize } from "@marblecore/serializer/lib/interface";
    /**
     * Deserializes data to a target.
     * @param pTarget Reference to the target.
     * @param pData Reference to the serialized data.
     */
    export function Deserialize<T, S extends ISerialize>(
        pTarget: T,
        pData: S
    ): T;
}

declare module "@marblecore/serializer/decorators/deserialize" {
    /** Decorator which marks a property to be deserialized. */
    export function deserialize<T>(
        pTarget: T,
        sProperty: string,
        pPropertyDescriptor?: PropertyDescriptor
    ): void;
}

declare module "@marblecore/serializer/decorators/deserializer" {
    /** Decorator which marks a function as the deserializer of an object. */
    export function deserializer<T>(
        pTarget: T,
        sProperty: string,
        pPropertyDescriptor?: PropertyDescriptor
    ): void;
}

declare module "@marblecore/serializer/decorators/serializer" {
    /** Decorator which marks a function as the serializer of an object. */
    export function serializer<T>(
        pTarget: T,
        sProperty: string,
        pPropertyDescriptor?: PropertyDescriptor
    ): void;
}

declare module "@marblecore/serializer/decorators/serialize" {
    /** Decorator which marks a property to be serialized. */
    export function serialize<T>(
        pTarget: T,
        sProperty: string,
        pPropertyDescriptor?: PropertyDescriptor
    ): void;
}
