/*! Tripetto Collector 0.4.23 - Copyright (C) 2018 Tripetto B.V. - All Rights Reserved */

/// <reference path="marblecore.functional.d.ts" />
/// <reference path="tripetto.slots.d.ts" />
/// <reference path="tripetto.vfsm.d.ts" />
/// <reference path="tripetto.map.d.ts" />
/// <reference path="marblecore.callback.d.ts" />
/// <reference path="marblecore.serializer.d.ts" />

declare module "tripetto-collector" {
    /** Dependencies */
    import * as NSFunctional from "@marblecore/functional";
    import * as NSSlots from "@tripetto/slots";
    /** Contains the package name. */
    export const Name: string;
    /** Contains the version number. */
    export const Version: string;
    /** Explicit exports. */
    export { Branch } from "tripetto-collector/lib/branches/branch";
    export { Cluster } from "tripetto-collector/lib/clusters/cluster";
    export { Collector } from "tripetto-collector/lib/collector";
    export { Condition } from "tripetto-collector/lib/conditions/condition";
    export {
        ConditionBlock
    } from "tripetto-collector/lib/conditions/blocks/block";
    export {
        ConditionBlocks
    } from "tripetto-collector/lib/conditions/blocks/blocks";
    export {
        IConditionBlockType,
        TConditionBlock
    } from "tripetto-collector/lib/conditions/blocks/type";
    export {
        Data,
        Store,
        Errors,
        Instance,
        IDataValues,
        ISnapshot
    } from "@tripetto/vfsm";
    export {
        ICondition,
        IConditionBlock,
        IMap,
        INode,
        INodeBlock
    } from "@tripetto/map";
    export { Node } from "tripetto-collector/lib/nodes/node";
    export { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    export { NodeBlocks } from "tripetto-collector/lib/nodes/blocks/blocks";
    export {
        INodeBlockType,
        TNodeBlock
    } from "tripetto-collector/lib/nodes/blocks/type";
    export { IObservableNode } from "tripetto-collector/lib/nodes/observable";
    export { Slot } from "@tripetto/slots";
    /** Decorators */
    export {
        condition
    } from "tripetto-collector/lib/conditions/blocks/decorator";
    export { node } from "tripetto-collector/lib/nodes/blocks/decorator";
    /** Slots storage framework. */
    export import Slots = NSSlots;
    /** Functional toolkit to make your life as a developer easier. */
    export import F = NSFunctional;
    /** Implicit exports */
    export * from "@marblecore/callback";
}

declare module "tripetto-collector/lib/branches/branch" {
    /** Dependencies */
    import { Branch as BranchFactory } from "@tripetto/vfsm";
    import { IBranch } from "@tripetto/map";
    import { Collector } from "tripetto-collector/lib/collector";
    import { Cluster } from "tripetto-collector/lib/clusters/cluster";
    export class Branch<Rendering = void> extends BranchFactory<IBranch> {
        /** Reference to the collector. */
        readonly Collector: Collector<Rendering>;
        /** Reference to the cluster. */
        readonly Cluster: Cluster<Rendering>;
        /**
         * Invoked when the branch condition is requested.
         * @return For now, the branch condition is always true.
         * @event
         */
        OnCondition(): boolean;
    }
}

declare module "tripetto-collector/lib/clusters/cluster" {
    import { Instance, State } from "@tripetto/vfsm";
    import { ICluster } from "@tripetto/map";
    import { Await } from "@marblecore/callback";
    import { Collector } from "tripetto-collector/lib/collector";
    export class Cluster<Rendering = void> extends State<ICluster> {
        /** Reference to the collector. */
        readonly Collector: Collector<Rendering>;
        /** Retrieves the progress of the form as a percentage. */
        readonly ProgressAsPercentage: number;
        /**
         * Invoked when the enter condition for the cluster is requested.
         * @event
         */
        OnEnterCondition(): boolean;
        /**
         * Invoked when the cluster is entered.
         * @param pInstance Reference to the instance.
         * @param pAwait Reference to the await pointer
         * @return Returns the await reference.
         * @event
         */
        OnEnter(pInstance: Instance, pAwait: Await): Await;
        /**
         * Invoked when the cluster leave condition is requested.
         * @event
         */
        OnLeaveCondition(): boolean;
        /**
         * Invoked when the cluster is left.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnLeave(pInstance: Instance): void;
        /**
         * Invoked when the cluster is canceled.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnCancel(pInstance: Instance): void;
        /**
         * Invoked when the cluster needs to be validated.
         * @event
         */
        OnValidate(): boolean;
        /**
         * Invoked when the cluster is validated.
         * @param sResult Specifies the validation state.
         * @param sType Specifies if the validation type.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnValidated(
            sResult: "fail" | "pass",
            sType: "initial" | "revalidate",
            pInstance: Instance
        ): void;
    }
}

declare module "tripetto-collector/lib/collector" {
    /** Dependencies */
    import { Instance, Ontology } from "@tripetto/vfsm";
    import { IMap } from "@tripetto/map";
    import { Await } from "@marblecore/callback";
    import { Cluster } from "tripetto-collector/lib/clusters/cluster";
    import { Node } from "tripetto-collector/lib/nodes/node";
    import { Branch } from "tripetto-collector/lib/branches/branch";
    import { Condition } from "tripetto-collector/lib/conditions/condition";
    import { IObservableNode } from "tripetto-collector/lib/nodes/observable";
    export class Collector<Rendering = void> extends Ontology<
        Cluster<Rendering>,
        Node<Rendering>,
        Branch<Rendering>,
        Condition<Rendering>,
        IMap
    > {
        /**
         * Creates a new collector instance.
         * @param pForm Specifies the form.
         * @param bVerbose Specifies if verbose mode should be enabled. This outputs
         * detailed information to the console.
         * @param fnRender Specifies a render function which is invoked when the collector needs to be rendered.
         * @param fnEnd Specifies a function which is invoked when an instance ends.
         */
        constructor(
            pForm: IMap | string,
            bVerbose?: boolean,
            fnRender?: (
                pInstance: Instance,
                pCollector: Collector<Rendering>
            ) => void,
            fnEnd?: (
                pInstance: Instance,
                sType: "ended" | "stopped" | "paused",
                pCollector: Collector<Rendering>
            ) => void
        );
        /** Contains the observable nodes for the first active instance. */
        readonly Nodes: IObservableNode<Rendering>[];
        /** Retrieves a reference to the most recent actionable observer of the first active instance. */
        readonly Observer: Await | undefined;
        /** Retrieves if the validation of the first active instance failed. */
        readonly IsValidationFailed: boolean;
        /** Retrieves if the validation of the first active instance passed. */
        readonly IsValidationPassed: boolean;
        /** Retrieves the most recent step direction of the first active instance. */
        readonly Direction: "forward" | "backward";
        /** Retrieves the progress percentage of the first active instance. */
        readonly ProgressPercentage: number;
        /**
         * Invoked when the rendering should be updated.
         * @param pInstance Reference to the instance.
         */
        protected OnInstanceRenderUpdate(pInstance: Instance): void;
        /**
         * Invoked when an instance is started.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnInstanceStart(pInstance: Instance): void;
        /**
         * Invoked when an instance is ended. This event is always invoked
         * when an instance ends normally, is stopped or paused.
         * @param pInstance Reference to the instance.
         * @param sType Specifies why the instance is ended. Can be one of the
         * following values:
         * - `ended`: The instance has ended normally;
         * - `break`: The instance was forced to stop;
         * - `paused`: The instance was paused.
         * @event
         */
        OnInstanceEnd(
            pInstance: Instance,
            sType: "ended" | "stopped" | "paused"
        ): void;
        /**
         * Invoked when a cluster or node should be rendered to a context.
         * @param pInstance Reference to the instance.
         * @param pReference Reference to the cluster or node to render.
         * @param pObserver Reference to the await pointer which can be used to give
         * observer feedback to the collector.
         */
        OnInstanceRender(
            pInstance: Instance,
            pReference: Cluster<Rendering> | Node<Rendering>,
            pObserver: Await
        ): void;
        /**
         * Invoked when the context should be cleared.
         * @param pInstance Reference to the instance.
         * @param sDirection Specifies the step direction.
         */
        OnInstanceUnrender(
            pInstance: Instance,
            sDirection: "forward" | "backward"
        ): void;
        /**
         * Invoked when the progress changes.
         * @param pInstance Reference to the instance.
         * @param nPercentage Contains the progress percentage.
         * @event
         */
        OnInstanceProgress(pInstance: Instance, nPercentage: number): void;
        /**
         * Invoked when the instance makes a step.
         * @param pInstance Reference to the instance.
         * @param sDirection Specifies the step direction.
         */
        OnInstanceStep(
            pInstance: Instance,
            sDirection: "forward" | "backward"
        ): void;
        /**
         * Invoked when the cluster is validated.
         * @param pInstance Reference to the instance.
         * @param sResult Specifies the validation state.
         * @param sType Specifies if the validation type.
         * @event
         */
        OnInstanceValidated(
            pInstance: Instance,
            sResult: "fail" | "pass",
            sType: "initial" | "revalidate"
        ): void;
    }
}

declare module "tripetto-collector/lib/conditions/condition" {
    /** Dependencies */
    import {
        Condition as ConditionFactory,
        Conditions,
        Instance
    } from "@tripetto/vfsm";
    import { ICondition } from "@tripetto/map";
    import { Callback } from "@marblecore/callback";
    import { Collector } from "tripetto-collector/lib/collector";
    export class Condition<Rendering = void> extends ConditionFactory<
        ICondition
    > {
        /**
         * Creates a new condition instance.
         * @param pConditions Reference to the conditions dispatcher.
         * @param pCondition Reference to the condition data.
         */
        constructor(pConditions: Conditions, pCondition: ICondition);
        /** Reference to the collector. */
        readonly Collector: Collector<Rendering>;
        /**
         * Invoked when the collector wants to verify the condition.
         * @param pInstance Reference to the instance.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous condition callback.
         * @return Returns the result of the condition or the callback reference if
         * the condition should be handled asynchronously. Returns `false` when no
         * block is available.
         * @event
         */
        OnCondition(
            pInstance: Instance,
            pCallback: Callback<boolean>
        ): boolean | Callback<boolean>;
    }
}

declare module "tripetto-collector/lib/conditions/blocks/block" {
    /** Dependencies */
    import { ICondition, IConditionBlock } from "@tripetto/map";
    import { Data, Instance } from "@tripetto/vfsm";
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Callback } from "@marblecore/callback";
    import { Node } from "tripetto-collector/lib/nodes/node";
    import { Cluster } from "tripetto-collector/lib/clusters/cluster";
    import { Branch } from "tripetto-collector/lib/branches/branch";
    import { Condition } from "tripetto-collector/lib/conditions/condition";
    import { Slot } from "@tripetto/slots";
    export abstract class ConditionBlock<
        Properties extends IConditionBlock = IConditionBlock
    > {
        /** Contains the type name of the block. */
        static readonly Type: string;
        /**
         * Creates a new condition block instance.
         * @param pCondition Reference to the condition.
         */
        constructor(pCondition: Condition);
        /** Retreieves the condition. */
        protected readonly Condition: Condition;
        /** Contains the condition properties. */
        protected readonly Meta: ICondition;
        /** Retrieves the block properties. */
        protected readonly Props: Properties;
        /** Retrieves the node (if any). */
        protected readonly Node: Node | undefined;
        /** Retrieves the cluster (if any). */
        protected readonly Cluster: Cluster | undefined;
        /** Retrieves the branch (if any). */
        protected readonly Branch: Branch | undefined;
        /**
         * Retrieves a slot.
         * @param pInstance Reference the instance.
         * @return Returns the slot or `undefined` if the slot is not found.
         */
        protected Slot<T extends TSerializeTypes, S extends Slot<T> = Slot<T>>(
            pInstance: Instance
        ): Slot | undefined;
        /**
         * Retrieves a slot and throws an error if the slot is not found.
         * @param pInstance Reference the instance.
         * @return Returns the slot or `undefined` if the slot is not found.
         */
        protected SlotAssert<
            T extends TSerializeTypes,
            S extends Slot<T> = Slot<T>
        >(pInstance: Instance): Slot;
        /**
         * Retrieves the data slot.
         * @param pInstance Reference the instance.
         * @return Returns a reference to the data slot or `undefined` if the slot is not found.
         */
        protected Data<T extends TSerializeTypes>(
            pInstance: Instance
        ): Data<T> | undefined;
        /**
         * Retrieves the data slot.
         * @param pInstance Reference the instance.
         * @return Returns a reference to the data slot or `undefined` if the slot is not found.
         */
        protected DataAssert<T extends TSerializeTypes>(
            pInstance: Instance
        ): Data<T>;
        /**
         * Invoked when the block should perform the condition.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous condition callback.
         * @param pInstance Reference to the instance.
         * @return Returns the result of the condition or the callback reference if
         * the condition should be handled asynchronously.
         * @event
         */
        abstract OnCondition(
            pInstance: Instance,
            pCallback: Callback<boolean>
        ): boolean | Callback<boolean>;
    }
}

declare module "tripetto-collector/lib/conditions/blocks/blocks" {
    import { TConditionBlock } from "tripetto-collector/lib/conditions/blocks/type";
    import { ConditionBlock } from "tripetto-collector/lib/conditions/blocks/block";
    import { Condition } from "tripetto-collector/lib/conditions/condition";
    export class ConditionBlocks {
        /** Retrieves a list of registered blocks. */
        static readonly All: TConditionBlock[];
        /**
         * Registers a block.
         * @param pBlock Specifies the block.
         * @param sType Specifies the block type name.
         */
        static Register(pBlock: TConditionBlock, sType: string): void;
        /**
         * Retrieves the block constructor of the specified block type.
         * @param sType Specifies the block type name.
         * @return Returns the block constructor or `undefined` if the block
         * type is not available.
         */
        static OfType(sType: string): TConditionBlock | undefined;
        /**
         * Provides a new instance of a condition block for the supplied condition.
         * @param pCondition Reference to the condition.
         * @return Returns a condition block instance.
         */
        static For<Rendering>(
            pCondition: Condition<Rendering>
        ): ConditionBlock | undefined;
    }
}

declare module "tripetto-collector/lib/conditions/blocks/type" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { ConditionBlock } from "tripetto-collector/lib/conditions/blocks/block";
    import { Condition } from "tripetto-collector/lib/conditions/condition";
    export interface IConditionBlockType<T extends ConditionBlock> {
        /** Contains the block type. */
        readonly Type: string;
        /**
         * Creates a new block instance.
         * @param pCondition Reference to the condition.
         */
        new (pCondition: Condition): T;
        /** Optional readonly properties. */
        readonly [sProperty: string]: F.TAny;
    }
    export type TConditionBlock = IConditionBlockType<ConditionBlock>;
}

declare module "tripetto-collector/lib/nodes/node" {
    /** Dependencies */
    import { Instance, Transducer, Transducers } from "@tripetto/vfsm";
    import { Await, Callback } from "@marblecore/callback";
    import { INode } from "@tripetto/map";
    import { Collector } from "tripetto-collector/lib/collector";
    import { Cluster } from "tripetto-collector/lib/clusters/cluster";
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    export class Node<Rendering = void> extends Transducer<INode> {
        /**
         * Creates a new node instance.
         * @param pNodes Reference to the nodes dispatcher.
         * @param pNode Reference to the node data.
         */
        constructor(pNodes: Transducers, pNode: INode);
        /** Reference to the collector. */
        readonly Collector: Collector<Rendering>;
        /** Reference to the cluster. */
        readonly Cluster: Cluster<Rendering>;
        /** Retrieves the block. */
        readonly Block: NodeBlock<Rendering> | undefined;
        /**
         * Invoked when the collector wants to verify the condition for the node.
         * This event is invoked just before the node is entered.
         * @param pInstance Reference to the instance.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous condition callback.
         * @return Returns the result of the condition or the callback reference if
         * the condition should be handled asynchronously.
         * @event
         */
        OnEnterCondition(
            pInstance: Instance,
            pCallback: Callback<boolean>
        ): boolean | Callback<boolean>;
        /**
         * Invoked when the node is entered.
         * @param pInstance Reference to the instance.
         * @param pAwait Reference to the await pointer.
         * @event
         */
        OnEnter(pInstance: Instance, pAwait: Await): Await;
        /**
         * Invoked when the node is left.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnLeave(pInstance: Instance): void;
        /**
         * Invoked when the transducer is canceled.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnCancel(pInstance: Instance): void;
        /**
         * Invoked when the node is validated.
         * @param pInstance Reference to the instance.
         * @param sCurrent Specifies the current node validation state.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous validation callback.
         * @param bObserved Specifies if the node is observed.
         * @return Returns the result of the validation or the callback reference if
         * the validation should be handled asynchronously.
         * @event
         */
        OnValidate(
            pInstance: Instance,
            sCurrent: "unknown" | "fail" | "pass",
            pCallback: Callback<boolean>,
            bObserved: boolean
        ): boolean | Callback<boolean>;
    }
}

declare module "tripetto-collector/lib/nodes/blocks/block" {
    /** Dependencies */
    import { INode, INodeBlock } from "@tripetto/map";
    import { Data, Instance } from "@tripetto/vfsm";
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Await, Callback } from "@marblecore/callback";
    import { Kinds, Slot } from "@tripetto/slots";
    import { Node } from "tripetto-collector/lib/nodes/node";
    export abstract class NodeBlock<
        Rendering = void,
        Properties extends INodeBlock = INodeBlock
    > {
        /** Contains the type name of the block. */
        static readonly Type: string;
        /**
         * Creates a new node block instance.
         * @param pNode Reference to the node.
         */
        constructor(pNode: Node<Rendering>);
        /** Reference to the node. */
        protected readonly Node: Node<Rendering>;
        /** Contains the node properties. */
        protected readonly Meta: INode;
        /** Contains the block properties. */
        protected readonly Props: Properties;
        /**
         * Retrieves a slot.
         * @param sReference Specifies the slot reference.
         * @param sKind Specifies the slot kind.
         * @return Returns the data or `undefined` if the slot is not found.
         */
        protected Slot<S extends Slot = Slot>(
            sReference: string,
            sKind?: Kinds
        ): Slot | undefined;
        /**
         * Retrieves a slot and throws an error if the slot is not found.
         * @param sReference Specifies the slot reference.
         * @param sKind Specifies the slot kind.
         * @return Returns the data or `undefined` if the slot is not found.
         */
        protected SlotAssert<S extends Slot = Slot>(
            sReference: string,
            sKind?: Kinds
        ): Slot;
        /**
         * Retrieves a slot data store.
         * @param pInstance Reference to the instance.
         * @param pReference Specifies the slot reference.
         * @param sKind Specifies the slot kind.
         * @return Returns the data or `undefined` if the slot is not found.
         */
        protected Data<T extends TSerializeTypes, S extends Slot<T> = Slot<T>>(
            pInstance: Instance,
            pReference: Slot | string,
            sKind?: Kinds
        ): Data<T> | undefined;
        /**
         * Retrieves a slot data store or throws an error if the slot is not found.
         * @param pInstance Reference to the instance.
         * @param sReference Specifies the slot reference.
         * @param sKind Specifies the slot kind.
         * @return Returns the data.
         */
        protected DataAssert<
            T extends TSerializeTypes,
            S extends Slot<T> = Slot<T>
        >(
            pInstance: Instance,
            pReference: Slot | string,
            sKind?: Kinds
        ): Data<T>;
        /**
         * Invoked when the enter condition for the block is requested.
         * @param pInstance Reference to the instance.
         * @param pCallback Reference to a callback pointer which can be used when
         * implementing asynchronous conditions.
         * @return Returns the result of the condition or the callback reference if
         * the condition should be handled asynchronously.
         * @event
         */
        OnCondition(
            pInstance: Instance,
            pCallback: Callback<boolean>
        ): boolean | Callback<boolean>;
        /**
         * Invoked when the block should be rendered.
         * @param pInstance Reference to the instance.
         * @param pObserver Reference to the observer which can be notified when
         * the block is ready.
         * @return Should return the rendering.
         * @event
         */
        abstract OnRender(pInstance: Instance, pObserver: Await): Rendering;
        /**
         * Invoked when the block is validated.
         * @param pInstance Reference to the instance.
         * @param sCurrent Specifies the current block validation state.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous validation callback.
         * @param bObserved Specifies if the block is observed.
         * @return Returns the result of the validation or the callback reference if
         * the validation should be handled asynchronously.
         * @event
         */
        abstract OnValidate(
            pInstance: Instance,
            sCurrent: "unknown" | "fail" | "pass",
            pCallback: Callback<boolean>,
            bObserved: boolean
        ): boolean | Callback<boolean>;
        /**
         * Destroys the block.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnDestroy(pInstance: Instance): void;
    }
}

declare module "tripetto-collector/lib/nodes/blocks/blocks" {
    import { TNodeBlock } from "tripetto-collector/lib/nodes/blocks/type";
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    import { Node } from "tripetto-collector/lib/nodes/node";
    export class NodeBlocks {
        /** Retrieves a list of registered blocks. */
        static readonly All: TNodeBlock[];
        /**
         * Registers the supplied block.
         * @param pBlock Specifies the block.
         * @param sType Specifies the block type name.
         */
        static Register<Rendering>(
            pBlock: TNodeBlock<Rendering>,
            sType: string
        ): void;
        /**
         * Retrieves the block constructor of the specified block type.
         * @param sType Specifies the block type name.
         * @return Returns the block constructor or `undefined` if the block
         * type is not available.
         */
        static OfType<Rendering>(
            sType: string
        ): TNodeBlock<Rendering> | undefined;
        /**
         * Provides a new instance of a condition block for the supplied condition.
         * @param pCondition Reference to the condition.
         * @return Returns a condition block instance.
         */
        static For<Rendering>(
            pNode: Node<Rendering>
        ): NodeBlock<Rendering> | undefined;
    }
}

declare module "tripetto-collector/lib/nodes/blocks/type" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    import { Node } from "tripetto-collector/lib/nodes/node";
    export interface INodeBlockType<Rendering, T extends NodeBlock<Rendering>> {
        /** Contains the block type. */
        readonly Type: string;
        /**
         * Creates a new block instance.
         * @param pNode Reference to the node.
         */
        new (pNode: Node): T;
        /** Optional readonly properties. */
        readonly [sProperty: string]: F.TAny;
    }
    export type TNodeBlock<Rendering = {} | void> = INodeBlockType<
        Rendering,
        NodeBlock<Rendering>
    >;
}

declare module "tripetto-collector/lib/nodes/observable" {
    /** Dependencies */
    import { Instance } from "@tripetto/vfsm";
    import { INode } from "@tripetto/map";
    import { Await } from "@marblecore/callback";
    import { Node } from "tripetto-collector/lib/nodes/node";
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    export interface IObservableNode<Rendering> {
        /** Contains a unique identifier for the node. */
        Id: string;
        /** Contains a unique key for the node. */
        Key: string;
        /** Reference to the node. */
        Node: Node<Rendering>;
        /** Reference to the node block. */
        Block: NodeBlock<Rendering> | undefined;
        /** Reference to the node properties. */
        Props: INode;
        /** Reference to the instance. */
        Instance: Instance;
        /** Reference to the node observer. */
        Observer: Await;
    }
}

declare module "tripetto-collector/lib/conditions/blocks/decorator" {
    /**
     * Registers a condition block.
     * @param sType Specifies the block type name.
     */
    export function condition<T>(sType: string): (pBlock: T) => void;
}

declare module "tripetto-collector/lib/nodes/blocks/decorator" {
    /**
     * Registers a node block.
     * @param sType Specifies the block type name.
     */
    export function node<T>(sType: string): (pBlock: T) => void;
}
