/*! Tripetto Collector 0.4.23 - Copyright (C) 2018 Tripetto B.V. - All Rights Reserved */

/// <reference path="marblecore.functional.d.ts" />
/// <reference path="marblecore.grid.d.ts" />
/// <reference path="marblecore.matrix.d.ts" />
/// <reference path="marblecore.metrics.d.ts" />
/// <reference path="marblecore.serializer.d.ts" />
/// <reference path="tripetto.slots.d.ts" />
/// <reference path="marblecore.hookup.d.ts" />

declare module "@tripetto/map" {
    /** Explicit exports */
    export { Branch } from "@tripetto/map/lib/branches/branch";
    export { Branches } from "@tripetto/map/lib/branches/branches";
    export { Cluster } from "@tripetto/map/lib/clusters/cluster";
    export { Clusters } from "@tripetto/map/lib/clusters/clusters";
    export { Condition } from "@tripetto/map/lib/conditions/condition";
    export { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    export {
        ConditionBlocks
    } from "@tripetto/map/lib/conditions/blocks/blocks";
    export { Conditions } from "@tripetto/map/lib/conditions/conditions";
    export { IBranch } from "@tripetto/map/lib/branches/interface";
    export { ICluster } from "@tripetto/map/lib/clusters/interface";
    export { ICondition } from "@tripetto/map/lib/conditions/interface";
    export {
        IConditionBlock
    } from "@tripetto/map/lib/conditions/blocks/interface";
    export {
        IConditionBlockType,
        TConditionBlock
    } from "@tripetto/map/lib/conditions/blocks/type";
    export { IMap } from "@tripetto/map/lib/interface";
    export { IMapEditor } from "@tripetto/map/lib/editor";
    export { INode } from "@tripetto/map/lib/nodes/interface";
    export { INodeBlock } from "@tripetto/map/lib/nodes/blocks/interface";
    export {
        INodeBlockType,
        TNodeBlock
    } from "@tripetto/map/lib/nodes/blocks/type";
    export { Map } from "@tripetto/map/lib/map";
    export { Node } from "@tripetto/map/lib/nodes/node";
    export { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    export { NodeBlocks } from "@tripetto/map/lib/nodes/blocks/blocks";
    export { Nodes } from "@tripetto/map/lib/nodes/nodes";
    export { TBranchCulling } from "@tripetto/map/lib/branches/culling";
    export { TBranchTerminator } from "@tripetto/map/lib/branches/terminator";
    export { IDefinition } from "@tripetto/map/lib/definition";
    /** Decorators */
    export { condition } from "@tripetto/map/lib/conditions/blocks/decorator";
    export { definition } from "@tripetto/map/lib/decorators/definition";
    export { name } from "@tripetto/map/lib/decorators/name";
    export { node } from "@tripetto/map/lib/nodes/blocks/decorator";
    /** Events */
    export {
        IBranchCullingEvent,
        IBranchTerminatorEvent
    } from "@tripetto/map/lib/branches/events";
    export {
        IConditionBlockAssignEvent,
        IConditionBlockChangeEvent,
        IConditionBlockUnassignEvent
    } from "@tripetto/map/lib/conditions/events";
    export {
        INodeBlockAssignEvent,
        INodeBlockChangeEvent,
        INodeBlockUnassignEvent
    } from "@tripetto/map/lib/nodes/events";
    /** Default export */
    export { Map as default } from "@tripetto/map/lib/map";
}

declare module "@tripetto/map/lib/branches/branch" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { GridBranch } from "@marblecore/grid";
    import { IItemChangeEvent } from "@marblecore/matrix";
    import { IPoint } from "@marblecore/metrics";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { Clusters } from "@tripetto/map/lib/clusters/clusters";
    import { Condition } from "@tripetto/map/lib/conditions/condition";
    import { Conditions } from "@tripetto/map/lib/conditions/conditions";
    import { Map } from "@tripetto/map/lib/map";
    import { Node } from "@tripetto/map/lib/nodes/node";
    import { Branches } from "@tripetto/map/lib/branches/branches";
    import { TBranchCulling } from "@tripetto/map/lib/branches/culling";
    import { TBranchEvents } from "@tripetto/map/lib/branches/events";
    import { IBranch } from "@tripetto/map/lib/branches/interface";
    import { TBranchTerminator } from "@tripetto/map/lib/branches/terminator";
    export class Branch extends GridBranch<Clusters, Node, TBranchEvents> {
        /** Retrieves if the branch is validating. */
        static readonly IsValidating: boolean;
        /**
         * Creates a new branch instance.
         * @param pBranches Reference to the branch dispatcher.
         */
        constructor(pBranches: Branches);
        /** Retrieves the parent branch dispatcher. */
        readonly Parent: Branches | undefined;
        /** Retrieves the predecessor of the branch. */
        readonly Predecessor: Branch | undefined;
        /** Retrieves the successor of the branch. */
        readonly Successor: Branch | undefined;
        /** Retrieves the parent map. */
        readonly Map: Map | undefined;
        /** Retrieves the parent cluster. */
        readonly Cluster: Cluster | undefined;
        /** Retrieves the conditions. */
        readonly Conditions: Conditions;
        /** Retrieves the clusters. */
        readonly Clusters: Clusters;
        /** Retrieves the branch spacing. */
        readonly Spacing: number;
        /** Sets the offset for the branch. */
        Offset: number;
        /** Sets the culling mode. */
        Culling: TBranchCulling;
        /** Sets the termination mode for the branch. */
        Terminator: TBranchTerminator;
        /** Sets the jump-to-cluster. */
        Jump: Cluster | undefined;
        /** Sets the name of the branch. */
        Name: string;
        /**
         * Fired when a branch is changed.
         * @param nReason Specifies the reason for the change.
         * @event
         */
        protected OnChange(pEvent: IItemChangeEvent<this>): void;
        /**
         * Fired when a branch move occurs.
         * @param nMoveX Specifies the horizontal movement.
         * @param nMoveY Specifies the vertical movement.
         * @param pOffset Specifies the offset of the touch point.
         * @event
         */
        OnMove(nMoveX: number, nMoveY: number, pOffset: IPoint): void;
        /**
         * Iterate through each cluster and condition of the branch.
         * @param fnResource Specifies the resource function.
         * @param fnResource.pResource Contains a reference to the resource.
         * @return Returns `true` if the iteration is stopped (break).
         */
        ForEachDescendant(
            fnResource: (pResource: Condition | Cluster) => boolean | void
        ): boolean;
        /**
         * Applies the spacing for the conditions.
         * @param nWidth Specifies the width of the resource.
         * @param nHeight Specifies the height of the resource.
         * @return Returns `true` if the resource is resized.
         */
        Resize(nWidth?: number, nHeight?: number): boolean;
        /**
         * Generates a serialized representation of a branch.
         * @return Returns a reference to the serialized branch.
         */
        Serialize(): IBranch;
        /**
         * Deserializes to a branch.
         * @param pBranch Contains the serialized data.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @return Returns a reference to the instance.
         */
        Deserialize(pBranch: IBranch, pAfterwards: F.Stack): this;
        /**
         * Inserts a new branch before this branch.
         * @return Returns a reference to the new branch.
         */
        InsertBefore(): Branch;
        /**
         * Inserts a new branch after this branch.
         * @return Returns a reference to the new branch.
         */
        InsertAfter(): Branch;
        /** Deletes this branch from the map. */
        Delete(): this;
        /** Resets the branch. */
        Reset(): void;
        /** Destroys a branch. */
        Destroy(): void;
    }
}

declare module "@tripetto/map/lib/branches/branches" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { GridBranches } from "@marblecore/grid";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { Node } from "@tripetto/map/lib/nodes/node";
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { IBranchConstructor } from "@tripetto/map/lib/branches/constructor";
    import { IBranch } from "@tripetto/map/lib/branches/interface";
    export class Branches extends GridBranches<Branch, Node> {
        constructor(pBranchConstructor: IBranchConstructor, pCluster: Cluster);
        /** Retrieves the cluster of the branch dispatcher. */
        readonly Cluster: Cluster;
        /**
         * Invoked when the branch dimensions are changed. This aligns the space
         * occupied by the conditions.
         * @event
         */
        OnDimensions(): void;
        /**
         * Appends a new branch.
         * @return Returns the new branch.
         */
        Append(): Branch;
        /**
         * Inserts a new branch.
         * @return Returns the new branch.
         */
        Insert(): Branch;
        /**
         * Inserts a new branch before the supplied branch.
         * @param pBranch Specifies the reference branch.
         * @return Returns the new branch.
         */
        InsertBefore(pBranch: Branch): Branch;
        /**
         * Inserts a new branch after the supplied branch.
         * @param pBranch Specifies the reference branch.
         * @return Returns the new branch.
         */
        InsertAfter(pBranch: Branch): Branch;
        /**
         * Deserializes the supplied serialized branches to the branch dispatcher.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @param pBranches Specifies the branches.
         * @return Returns a reference to the instance.
         */
        Deserialize(pAfterwards: F.Stack, ...pBranches: IBranch[]): this;
    }
}

declare module "@tripetto/map/lib/clusters/cluster" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { GridCluster } from "@marblecore/grid";
    import { IItemPopEvent } from "@marblecore/matrix";
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { Branches } from "@tripetto/map/lib/branches/branches";
    import { Map } from "@tripetto/map/lib/map";
    import { Node } from "@tripetto/map/lib/nodes/node";
    import { Nodes } from "@tripetto/map/lib/nodes/nodes";
    import { Clusters } from "@tripetto/map/lib/clusters/clusters";
    import { TClusterEvents } from "@tripetto/map/lib/clusters/events";
    import { ICluster } from "@tripetto/map/lib/clusters/interface";
    export class Cluster extends GridCluster<Nodes, Node, TClusterEvents> {
        /**
         * Creates a new cluster instance.
         * @param pClusters Reference to the cluster dispatcher.
         */
        constructor(pClusters: Clusters);
        /** Retrieves the parent cluster dispatcher. */
        readonly Parent: Clusters | undefined;
        /** Retrieves the predecessor of the cluster. */
        readonly Predecessor: Cluster | undefined;
        /** Retrieves the successor of the cluster. */
        readonly Successor: Cluster | undefined;
        /** Retrieves the parent map. */
        readonly Map: Map | undefined;
        /** Retrieves the parent branch. */
        readonly Branch: Branch | undefined;
        /** Retrieves the nodes. */
        readonly Nodes: Nodes;
        /** Retrieves the branches. */
        readonly Branches: Branches;
        /** Retrieves if the cluster has branches. */
        readonly HasBranches: boolean;
        /** Sets the name of the cluster. */
        Name: string;
        /** Retrieves a list of branches which have a jump to this cluster. */
        readonly Jumps: Branch[];
        /**
         * Invoked when the cluster is popped.
         * @param pEvent Contains the event information.
         * @event
         */
        OnPop(pEvent: IItemPopEvent<this>): void;
        /**
         * Iterate through each node and branch of the cluster.
         * @param fnResource Specifies the resource function.
         * @param fnResource.pResource Contains a reference to the resource.
         * @return Returns `true` if the iteration is stopped (break).
         */
        ForEachDescendant(
            fnResource: (pResource: Node | Branch) => boolean | void
        ): boolean;
        /**
         * Calculates the dimensions of the cluster (only necessary when we are not moving).
         * @param nWidth Specifies the width of the cluster.
         * @param nHeight Specifies the height of the cluster.
         * @return Returns `true` if the cluster is resized.
         */
        Resize(nWidth?: number, nHeight?: number): boolean;
        /**
         * Generates a serialized representation of a cluster.
         * @return Returns a reference to the serialized cluster.
         */
        Serialize(): ICluster;
        /**
         * Deserializes to a cluster.
         * @param pCluster Contains the serialized cluster.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @return Returns a reference to the instance.
         */
        Deserialize(pCluster: ICluster, pAfterwards: F.Stack): this;
        /**
         * Inserts a new cluster before this cluster.
         * @return Returns a reference to the new cluster.
         */
        InsertBefore(): Cluster;
        /**
         * Inserts a new cluster after this cluster.
         * @return Returns a reference to the new cluster.
         */
        InsertAfter(): Cluster;
        /** Deletes this cluster from the map. */
        Delete(): this;
        /** Resets the cluster. */
        Reset(): void;
        /** Destroys a cluster. */
        Destroy(): void;
    }
}

declare module "@tripetto/map/lib/clusters/clusters" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { GridClusters } from "@marblecore/grid";
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { Node } from "@tripetto/map/lib/nodes/node";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { IClusterConstructor } from "@tripetto/map/lib/clusters/constructor";
    import { ICluster } from "@tripetto/map/lib/clusters/interface";
    export class Clusters extends GridClusters<Cluster, Node> {
        constructor(pClusterConstructor: IClusterConstructor, pBranch: Branch);
        /** Retrieves the branch of the cluster dispatcher. */
        readonly Branch: Branch;
        /**
         * Appends a new cluster.
         * @return Returns the new cluster.
         */
        Append(): Cluster;
        /**
         * Inserts a new cluster.
         * @return Returns the new cluster.
         */
        Insert(): Cluster;
        /**
         * Inserts a new cluster before the supplied cluster.
         * @param pCluster Specifies the reference cluster.
         * @return Returns the new cluster.
         */
        InsertBefore(pCluster: Cluster): Cluster;
        /**
         * Inserts a new cluster after the supplied cluster.
         * @param pCluster Specifies the reference cluster.
         * @return Returns the new branch.
         */
        InsertAfter(pCluster: Cluster): Cluster;
        /**
         * Deserializes the supplied serialized clusters to the cluster dispatcher.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @param pClusters Specifies the clusters.
         * @return Returns a reference to the instance.
         */
        Deserialize(pAfterwards: F.Stack, ...pClusters: ICluster[]): this;
    }
}

declare module "@tripetto/map/lib/conditions/condition" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { GridNode } from "@marblecore/grid";
    import { IItemChangeEvent } from "@marblecore/matrix";
    import { IPoint } from "@marblecore/metrics";
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { Map } from "@tripetto/map/lib/map";
    import { Conditions } from "@tripetto/map/lib/conditions/conditions";
    import { TConditionEvents } from "@tripetto/map/lib/conditions/events";
    import { ICondition } from "@tripetto/map/lib/conditions/interface";
    import { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    import { IConditionBlockType } from "@tripetto/map/lib/conditions/blocks/type";
    export class Condition extends GridNode<TConditionEvents> {
        /**
         * Creates a new condition instance.
         * @param pConditions Reference to the condition dispatcher.
         */
        constructor(pConditions: Conditions);
        /** Retrieves the parent condition dispatcher. */
        readonly Parent: Conditions | undefined;
        /** Retrieves the predecessor of the condition. */
        readonly Predecessor: Condition | undefined;
        /** Retrieves the successor of the condition. */
        readonly Successor: Condition | undefined;
        /** Retrieves the parent map. */
        readonly Map: Map | undefined;
        /** Retrieves the collection. */
        readonly Branch: Branch | undefined;
        /** Retrieves the parent cluster of the condition. */
        readonly Cluster: Cluster | undefined;
        /** Retrieves the block for the condition. */
        readonly Block: ConditionBlock | undefined;
        /** Retrieves the block type as a string. */
        readonly BlockType: string;
        /** Retrieves if the condition has a block. */
        readonly HasBlock: boolean;
        /** Retrieves if the block is invalid. */
        readonly IsBlockInvalid: boolean;
        /** Specifies if the condition is disabled. */
        Disabled: boolean | undefined;
        /** Retrieves the condition type. */
        readonly Type: IConditionBlockType<ConditionBlock> | undefined;
        /** Retrieves the condition name. */
        readonly Name: string;
        /**
         * Fired when the condition is changed.
         * @param pEvent Specifies the event payload.
         * @event
         */
        protected OnChange(pEvent: IItemChangeEvent<this>): void;
        /**
         * Fired when a condition move occurs.
         * @param nMoveX Specifies the horizontal movement.
         * @param nMoveY Specifies the vertical movement.
         * @param pOffset Specifies the offset of the touch point.
         * @event
         */
        OnMove(nMoveX: number, nMoveY: number, pOffset: IPoint): void;
        /**
         * Assigns a block to the condition.
         * @param pBlock Specifies the block type.
         * @return Returns a reference to the block instance.
         */
        Assign<T extends ConditionBlock>(
            pBlock: IConditionBlockType<T> | T
        ): T | undefined;
        /**
         * Unassigns a block.
         * @return Returns the previously assigned block or `undefined` if there
         * was none.
         */
        Unassign(): ConditionBlock | undefined;
        /**
         * Generates a serialized representation of a condition.
         * @return Returns a reference to the serialized condition.
         */
        Serialize(): ICondition;
        /**
         * Deserializes to a condition.
         * @param pCondition Contains the serialized condition.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @return Returns a reference to the instance.
         */
        Deserialize(pCondition: ICondition, pAfterwards: F.Stack): this;
        /**
         * Inserts a new condition before this condition.
         * @return Returns a reference to the new condition.
         */
        InsertBefore(): Condition;
        /**
         * Inserts a new condition after this condition.
         * @return Returns a reference to the new condition.
         */
        InsertAfter(): Condition;
        /** Enables the condition. */
        Enable(): this;
        /** Disables the condition. */
        Disable(): this;
        /** Deletes this condition from the map. */
        Delete(): this;
        /** Destroys the condition. */
        Destroy(): void;
    }
}

declare module "@tripetto/map/lib/conditions/blocks/block" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { ISerialize } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots";
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    import { Node } from "@tripetto/map/lib/nodes/node";
    import { Condition } from "@tripetto/map/lib/conditions/condition";
    import { TConditionBlockContext } from "@tripetto/map/lib/conditions/blocks/context";
    import { IConditionBlock } from "@tripetto/map/lib/conditions/blocks/interface";
    import { IConditionBlockType } from "@tripetto/map/lib/conditions/blocks/type";
    export abstract class ConditionBlock<
        T extends IConditionBlock = IConditionBlock
    > {
        /** Contains the type name for the block. */
        static readonly Type: string;
        /** Contains the localized name for the block. */
        static readonly Name: string;
        /** Contains the version of the block. */
        static readonly Version: string;
        /** Contains the block context. */
        static readonly Context: TConditionBlockContext;
        /**
         * Creates a new block instance or verifies an existing instance.
         * @param pBlock Reference to the block constructor or instance.
         * @param pCondition Reference to the parent condition.
         * @return Returns the block instance.
         */
        static Create<Block extends ConditionBlock>(
            pBlock: IConditionBlockType<Block> | Block | string,
            pCondition: Condition
        ): Block | undefined;
        /**
         * Constructs a new block.
         * @param pCondition Reference to the condition.
         */
        constructor(pCondition: Condition);
        /** Retrieves a reference to the condition. */
        readonly Condition: Condition;
        /** Retrieves the block type. */
        abstract readonly Type: IConditionBlockType<ConditionBlock>;
        /** Retrieves the block name. */
        readonly Name: string;
        /** Sets the node. */
        Node: Node | undefined;
        /** Sets the cluster. */
        Cluster: Cluster | undefined;
        /** Sets the branch. */
        Branch: Branch | undefined;
        /** Sets the slot. */
        Slot: Slot | undefined;
        /**
         * Invoked when the block is initialized. This is right after the
         * block instance is created and before deserialization occurs.
         * @event
         */
        protected OnInit(): void;
        /**
         * Invoked when a block version is outdated.
         * @param sOldVersion Specifies the old version which should be upgraded.
         */
        protected OnUpgrade(sOldVersion: string): void;
        /**
         * Invoked when the block of node is unassigned.
         * @param pNodeBlock Reference to the node block which is unassigned.
         * @return Should return `delete` if the condition should be deleted.
         */
        protected OnNodeBlockUnassign(
            pNodeBlock: NodeBlock
        ): "delete" | "remain";
        /**
         * Invoked when the block is assigned to a condition.
         * @param pPrevious Reference to the previous block which was attached to the condition.
         * @event
         */
        OnAssign(pPrevious: ConditionBlock | undefined): void;
        /**
         * Invoked when the block is unassigned from a condition.
         * @event
         */
        OnUnassign(): void;
        /**
         * Invoked when the block is destroyed.
         * @event
         */
        OnDestroy(): void;
        /**
         * Rerenders the block.
         * @param bShouldRerender Specifies if the condition should rerender.
         * @event
         */
        Rerender(bShouldRerender?: boolean): void;
        /**
         * Serializes a block.
         * @param pProperties Specifies additional serialized properties.
         * @return Returns a reference to the serialized block.
         */
        Serialize(pProperties?: ISerialize): T;
        /**
         * Deserializes to a block instance.
         * @param pBlock Contains the serialized block.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @return Returns a reference to the instance.
         */
        Deserialize(pBlock: T, pAfterwards: F.Stack): this;
        /** Unassignes the block from the condition. */
        Unassign(): void;
        /** Destroys the block. */
        Destroy(): void;
    }
}

declare module "@tripetto/map/lib/conditions/blocks/blocks" {
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    import { TConditionBlockContext } from "@tripetto/map/lib/conditions/blocks/context";
    import { TConditionBlock } from "@tripetto/map/lib/conditions/blocks/type";
    export class ConditionBlocks {
        /** Retrieves a list of registered blocks. */
        static readonly All: TConditionBlock[];
        /**
         * Registers a block.
         * @param pBlock Specifies the block.
         * @param sType Specifies the block type name.
         * @param sName Specifies the localized name for the block type.
         * @param pContext Specifies the context.
         * @param sVersion Specifies the version of the block in SemVer format (defaults to `0.0.0`).
         */
        static Register(
            pBlock: TConditionBlock,
            sType: string,
            sName: string,
            pContext: TConditionBlockContext,
            sVersion: string
        ): void;
        /**
         * Retrieves the registered blocks for the specified context.
         * @param pContext Specifies the context.
         */
        static For(
            pContext: TConditionBlockContext | NodeBlock
        ): TConditionBlock[];
        /**
         * Retrieves the block constructor of the specified block type.
         * @param sType Specifies the block type name.
         * @return Returns the block constructor or `undefined` if the block
         * type is not available.
         */
        static OfType(sType: string): TConditionBlock | undefined;
    }
}

declare module "@tripetto/map/lib/conditions/conditions" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { GridNodes } from "@marblecore/grid";
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { Condition } from "@tripetto/map/lib/conditions/condition";
    import { IConditionConstructor } from "@tripetto/map/lib/conditions/constructor";
    import { ICondition } from "@tripetto/map/lib/conditions/interface";
    export class Conditions extends GridNodes<Condition> {
        constructor(
            pConditionConstructor: IConditionConstructor,
            pBranch: Branch
        );
        /** Retrieves the branch of the condition dispatcher. */
        readonly Branch: Branch;
        /**
         * Invoked when the conditions dimensions are changed.
         * @event
         */
        OnDimensions(): void;
        /**
         * Appends a new condition.
         * @return Returns the new condition.
         */
        Append(): Condition;
        /**
         * Inserts a new condition.
         * @return Returns the new condition.
         */
        Insert(): Condition;
        /**
         * Inserts a new condition before the supplied condition.
         * @param pCondition Specifies the reference condition.
         * @return Returns the new condition.
         */
        InsertBefore(pCondition: Condition): Condition;
        /**
         * Inserts a new condition after the supplied condition.
         * @param pCondition Specifies the reference condition.
         * @return Returns the new condition.
         */
        InsertAfter(pCondition: Condition): Condition;
        /**
         * Deserializes the supplied serialized conditions to the condition dispatcher.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @param pConditions Specifies the conditions.
         * @return Returns a reference to the instance.
         */
        Deserialize(pAfterwards: F.Stack, ...pConditions: ICondition[]): this;
    }
}

declare module "@tripetto/map/lib/branches/interface" {
    /** Dependencies */
    import { IGridResource } from "@marblecore/grid";
    import { ICluster } from "@tripetto/map/lib/clusters/interface";
    import { ICondition } from "@tripetto/map/lib/conditions/interface";
    import { TBranchCulling } from "@tripetto/map/lib/branches/culling";
    export interface IBranch extends IGridResource {
        readonly Conditions?: ICondition[];
        readonly Clusters?: ICluster[];
        readonly Jump?: string;
        readonly Culling: TBranchCulling;
        readonly Name: string;
    }
}

declare module "@tripetto/map/lib/clusters/interface" {
    /** Dependencies */
    import { IGridResource } from "@marblecore/grid";
    import { IBranch } from "@tripetto/map/lib/branches/interface";
    import { INode } from "@tripetto/map/lib/nodes/interface";
    export interface ICluster extends IGridResource {
        readonly Branches?: IBranch[];
        readonly Nodes?: INode[];
        readonly Name: string;
    }
}

declare module "@tripetto/map/lib/conditions/interface" {
    /** Dependencies */
    import { IGridResource } from "@marblecore/grid";
    import { IConditionBlock } from "@tripetto/map/lib/conditions/blocks/interface";
    export interface ICondition extends IGridResource {
        readonly Block?: IConditionBlock;
        readonly Disabled: boolean;
    }
}

declare module "@tripetto/map/lib/conditions/blocks/interface" {
    /** Dependencies */
    import { IDefinition } from "@tripetto/map/lib/definition";
    export interface IConditionBlock extends IDefinition {
        /** Contains the condition block type. */
        readonly Type: string;
        /** Contains the version of the block. */
        readonly Version: string;
        /** Contains the hash of an attached node. */
        readonly Node?: string;
        /** Contains the hash of an attached cluster. */
        readonly Cluster?: string;
        /** Contains the hash of an attached branch. */
        readonly Branch?: string;
        /** Contains the hash of an attached slot. */
        readonly Slot?: string;
    }
}

declare module "@tripetto/map/lib/conditions/blocks/type" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { Condition } from "@tripetto/map/lib/conditions/condition";
    import { TConditionBlockContext } from "@tripetto/map/lib/conditions/blocks/context";
    import { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    export interface IConditionBlockType<T extends ConditionBlock> {
        /** Contains the block type. */
        readonly Type: string;
        /** Contains the block name. */
        readonly Name: string;
        /** Contains the block context. */
        readonly Context: TConditionBlockContext;
        /**
         * Creates a new block instance.
         * @param pCondition Reference to the condition.
         */
        new (pCondition: Condition): T;
        /** Optional readonly properties. */
        readonly [sProperty: string]: F.TAny;
    }
    export type TConditionBlock = IConditionBlockType<ConditionBlock>;
}

declare module "@tripetto/map/lib/interface" {
    /** Dependencies */
    import { IGridResource } from "@marblecore/grid";
    import { ICluster } from "@tripetto/map/lib/clusters/interface";
    import { IMapEditor } from "@tripetto/map/lib/editor";
    export interface IMap extends IGridResource {
        /** Contains the map name. */
        readonly Name: string;
        /** Contains the clusters. */
        readonly Clusters: ICluster[];
        /** Contains information abouw the editor. */
        readonly Editor: IMapEditor;
    }
}

declare module "@tripetto/map/lib/editor" {
    /** Dependencies */
    import { ISerialize } from "@marblecore/serializer";
    export interface IMapEditor extends ISerialize {
        /** Editor name. */
        Name: string;
        /** Editor version. */
        Version: string;
    }
}

declare module "@tripetto/map/lib/nodes/interface" {
    /** Dependencies */
    import { IGridResource } from "@marblecore/grid";
    import { ISlot } from "@tripetto/slots";
    import { INodeBlock } from "@tripetto/map/lib/nodes/blocks/interface";
    export interface INode extends IGridResource {
        readonly Block?: INodeBlock;
        readonly Slots?: ISlot[];
        readonly Disabled: boolean;
        readonly Name: string;
        readonly NameVisible: boolean;
        readonly Placeholder: string;
        readonly Description: string;
        readonly Explanation: string;
    }
}

declare module "@tripetto/map/lib/nodes/blocks/interface" {
    /** Dependencies */
    import { IDefinition } from "@tripetto/map/lib/definition";
    export interface INodeBlock extends IDefinition {
        /** Contains the node block type. */
        readonly Type: string;
        /** Contains the version of the block. */
        readonly Version: string;
    }
}

declare module "@tripetto/map/lib/nodes/blocks/type" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { Node } from "@tripetto/map/lib/nodes/node";
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    export interface INodeBlockType<T extends NodeBlock> {
        /** Contains the block type. */
        readonly Type: string;
        /** Contains the block name. */
        readonly Name: string;
        /** Contains the block version. */
        readonly Version: string;
        /** Contains if the block is headless. */
        readonly IsHeadless: boolean;
        /**
         * Creates a new block instance.
         * @param pNode Reference to the node.
         */
        new (pNode: Node): T;
        /** Optional readonly properties. */
        readonly [sProperty: string]: F.TAny;
    }
    export type TNodeBlock = INodeBlockType<NodeBlock>;
}

declare module "@tripetto/map/lib/map" {
    import { Grid, GridRenderer } from "@marblecore/grid";
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { Clusters } from "@tripetto/map/lib/clusters/clusters";
    import { Condition } from "@tripetto/map/lib/conditions/condition";
    import { IMapEditor } from "@tripetto/map/lib/editor";
    import { IMap } from "@tripetto/map/lib/interface";
    import { Node } from "@tripetto/map/lib/nodes/node";
    export class Map extends Grid<Clusters, Node, "OnNameChange"> {
        /** Contains the name of the map. */
        Name: string;
        /**
         * Creates a new map instance.
         * @param pRenderer Reference to a renderer for the map.
         * @param pEditor Specifies information about the editor.
         */
        static Create(pRenderer?: GridRenderer<Map>, pEditor?: IMapEditor): Map;
        /**
         * Creates a new map instance.
         * @param pRenderer Reference to a renderer for the map.
         * @param pEditor Specifies information about the editor.
         */
        constructor(pRenderer?: GridRenderer<Map>, pEditor?: IMapEditor);
        /** Retrieves information about the editor application. */
        readonly Application: IMapEditor;
        /** Retrieves information about the editor used to create the definition. */
        readonly Editor: IMapEditor;
        /** Retrieves the clusters. */
        readonly Clusters: Clusters;
        /** Sets the map definition. */
        Definition: IMap;
        /**
         * Generates a serialized representation of a map.
         * @return Returns a reference to the serialized map.
         */
        Serialize(): IMap;
        /**
         * Deserializes to a map.
         * @param pMap Specifies the serialized map.
         * @return Returns a reference to the instance.
         */
        Deserialize(pMap: IMap): this;
        /**
         * Iterate recursively through each item in the map.
         * @param fnEach Callback which is invoked for each item.
         * @param sType Specifies the type of items.
         * @return Returns `true` if the iteration was interrupted.
         */
        ForEach(
            fnItem:
                | ((pItem: Branch, nDepth: number) => boolean | void)
                | ((pItem: Cluster, nDepth: number) => boolean | void)
                | ((pItem: Node, nDepth: number) => boolean | void)
                | ((pItem: Condition, nDepth: number) => boolean | void)
                | ((
                      pItem: Branch | Cluster | Node | Condition,
                      nDepth: number
                  ) => boolean | void),
            sType?: "branches" | "clusters" | "nodes" | "conditions" | "all",
            pExclude?: Branch | Cluster | Node | Condition
        ): boolean;
    }
}

declare module "@tripetto/map/lib/nodes/node" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { GridNode } from "@marblecore/grid";
    import { Slots } from "@tripetto/slots";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { Map } from "@tripetto/map/lib/map";
    import { TNodeEvents } from "@tripetto/map/lib/nodes/events";
    import { INode } from "@tripetto/map/lib/nodes/interface";
    import { Nodes } from "@tripetto/map/lib/nodes/nodes";
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    import {
        INodeBlockType,
        TNodeBlock
    } from "@tripetto/map/lib/nodes/blocks/type";
    export type TNodeType =
        | "undefined"
        | "static"
        | "note"
        | "error"
        | TNodeBlock;
    export class Node extends GridNode<TNodeEvents> {
        /** Contains the explanation for the node. */
        Explanation: string;
        /** Retrieves the parent node dispatcher. */
        readonly Parent: Nodes | undefined;
        /** Retrieves the predecessor of the node. */
        readonly Predecessor: Node | undefined;
        /** Retrieves the successor of the node. */
        readonly Successor: Node | undefined;
        /** Retrieves the parent map. */
        readonly Map: Map | undefined;
        /** Retrieves the parent cluster. */
        readonly Cluster: Cluster | undefined;
        /** Retrieves the block for the node. */
        readonly Block: NodeBlock | undefined;
        /** Retrieves the block type as a string. */
        readonly BlockType: string;
        /** Retrieves if the node has a block. */
        readonly HasBlock: boolean;
        /** Retrieves if the block is invalid. */
        readonly IsBlockInvalid: boolean;
        /** Retrieves the slots for the node. */
        readonly Slots: Slots;
        /** Specifies if the node is disabled. */
        Disabled: boolean | undefined;
        /** Retrieves the node type. */
        readonly Type: TNodeType;
        /** Sets the name of the node. */
        Name: string;
        /** Specifies if the name of the node should be visible. */
        NameVisible: boolean;
        /** Sets the description of the node. */
        Placeholder: string;
        /** Sets the description of the node. */
        Description: string;
        /** Retrieves the node label. This is the name of the node or the placeholder if no name is specified. */
        readonly Label: string;
        /**
         * Invokes when a branch is pushed or popped.
         * @event
         */
        OnPushOrPop(): void;
        /**
         * Calculates the dimensions of the node (only necessary when we are not moving).
         * @param nWidth Specifies the width of the node.
         * @param nHeight Specifies the height of the node.
         * @return Returns `true` if the node is resized.
         */
        Resize(nWidth?: number, nHeight?: number): boolean;
        /**
         * Calculates the z-index of the resource.
         * @return Returns `true` if the z-index is changed.
         */
        ZIndex(): boolean;
        /**
         * Assigns a block to the node.
         * @param pBlock Specifies the block type.
         * @return Returns a reference to the block instance.
         */
        Assign<T extends NodeBlock>(
            pBlock: INodeBlockType<T> | T
        ): T | undefined;
        /**
         * Unassigns a block.
         * @return Returns the previously assigned block or `undefined` if there
         * was none.
         */
        Unassign(): NodeBlock | undefined;
        /**
         * Generates a serialized representation of a node.
         * @return Returns a reference to the serialized data.
         */
        Serialize(): INode;
        /**
         * Deserializes to a node.
         * @param pNode Contains the serialized node.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @return Returns a reference to the instance.
         */
        Deserialize(pNode: INode, pAfterwards: F.Stack): this;
        /**
         * Inserts a new node before this node.
         * @return Returns a reference to the new node.
         */
        InsertBefore(): Node;
        /**
         * Inserts a new node after this node.
         * @return Returns a reference to the new node.
         */
        InsertAfter(): Node;
        /** Enables the node. */
        Enable(): this;
        /** Disables the node. */
        Disable(): this;
        /** Deletes this node from the map. */
        Delete(): this;
        /** Destroys the node. */
        Destroy(): void;
    }
}

declare module "@tripetto/map/lib/nodes/blocks/block" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { ISerialize } from "@marblecore/serializer";
    import { Slots } from "@tripetto/slots";
    import { Node } from "@tripetto/map/lib/nodes/node";
    import { INodeBlock } from "@tripetto/map/lib/nodes/blocks/interface";
    import { INodeBlockType } from "@tripetto/map/lib/nodes/blocks/type";
    export abstract class NodeBlock<T extends INodeBlock = INodeBlock> {
        /** Contains the type name of the block. */
        static readonly Type: string;
        /** Contains the localized name of the block. */
        static readonly Name: string;
        /** Contains the version of the block. */
        static readonly Version: string;
        /** Contains if the block is headless. */
        static readonly IsHeadless: boolean;
        /**
         * Creates a new block instance or verifies an existing instance.
         * @param pBlock Reference to the block constructor or instance.
         * @param pNode Reference to the parent node.
         * @return Returns the block instance.
         */
        static Create<Block extends NodeBlock>(
            pBlock: INodeBlockType<Block> | Block | string,
            pNode: Node
        ): Block | undefined;
        /**
         * Constructs a new block.
         * @param pNode Reference to the node.
         */
        constructor(pNode: Node);
        /** Retrieves a reference to the node. */
        readonly Node: Node;
        /** Retrieves a reference to the slots. */
        readonly Slots: Slots;
        /** Retrieves the block type. */
        abstract readonly Type: INodeBlockType<NodeBlock>;
        /** Retrieves the block name. */
        readonly Name: string;
        /**
         * Invoked when the block is initialized. This is right after the
         * block instance is created and before deserialization occurs.
         * @event
         */
        protected OnInit(): void;
        /**
         * Invoked when a block version is outdated.
         * @param sOldVersion Specifies the old version which should be upgraded.
         */
        protected OnUpgrade(sOldVersion: string): void;
        /**
         * Invoked when the block slots can be initialized. It is invoked after
         * the `OnInit` event.
         * @event
         */
        abstract OnSlots(): void;
        /**
         * Invoked when the block is assigned to a node.
         * @param pPrevious Reference to the previous block which was attached to the node.
         * @event
         */
        OnAssign(pPrevious: NodeBlock | undefined): void;
        /**
         * Invoked when the block is unassigned from a node.
         * @event
         */
        OnUnassign(): void;
        /**
         * Invoked when the block is destroyed.
         * @event
         */
        OnDestroy(): void;
        /**
         * Rerenders the block.
         * @param bShouldRerender Specifies if the node should rerender.
         * @event
         */
        Rerender(bShouldRerender?: boolean): void;
        /**
         * Serializes a block.
         * @param pProperties Specifies additional serialized properties.
         * @return Returns a reference to the serialized block.
         */
        Serialize(pProperties?: ISerialize): T;
        /**
         * Deserializes to a block instance.
         * @param pBlock Contains the serialized block.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @return Returns a reference to the instance.
         */
        Deserialize(pBlock: T, pAfterwards: F.Stack): this;
        /** Unassignes the block from the node. */
        Unassign(): void;
        /** Destroys the block. */
        Destroy(): void;
    }
}

declare module "@tripetto/map/lib/nodes/blocks/blocks" {
    import { TNodeBlock } from "@tripetto/map/lib/nodes/blocks/type";
    export class NodeBlocks {
        /** Retrieves a list of registered blocks. */
        static readonly All: TNodeBlock[];
        /**
         * Registers the supplied block.
         * @param pBlock Specifies the block.
         * @param sType Specifies the block type name.
         * @param sName Specifies the localized name for the block.
         * @param sVersion Specifies the version of the block in SemVer format (defaults to `0.0.0`).
         * @param sKind Specifies the block kind. Can be one of the following values:
         * - `headless`: Specifies the block is headless (has no user interface);
         * - `ui`: Specifies the block implements a user interface.
         */
        static Register(
            pBlock: TNodeBlock,
            sType: string,
            sName: string,
            sVersion: string,
            sKind: "headless" | "ui"
        ): void;
        /**
         * Retrieves the block constructor of the specified block type.
         * @param sType Specifies the block type name.
         * @return Returns the block constructor or `undefined` if the block
         * type is not available.
         */
        static OfType(sType: string): TNodeBlock | undefined;
    }
}

declare module "@tripetto/map/lib/nodes/nodes" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { GridNodes } from "@marblecore/grid";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { INodeConstructor } from "@tripetto/map/lib/nodes/constructor";
    import { INode } from "@tripetto/map/lib/nodes/interface";
    import { Node } from "@tripetto/map/lib/nodes/node";
    export class Nodes extends GridNodes<Node> {
        constructor(pNodeConstructor: INodeConstructor, pCluster: Cluster);
        /** Retrieves the cluster of the node dispatcher. */
        readonly Cluster: Cluster;
        /**
         * Inserts a new node.
         * @return Returns the new node.
         */
        Append(): Node;
        /**
         * Inserts a new node.
         * @return Returns the new node.
         */
        Insert(): Node;
        /**
         * Inserts a new node before the supplied node.
         * @param pNode Specifies the reference node.
         * @return Returns the new node.
         */
        InsertBefore(pNode: Node): Node;
        /**
         * Inserts a new node after the supplied node.
         * @param pNode Specifies the reference node.
         * @return Returns the new node.
         */
        InsertAfter(pNode: Node): Node;
        /**
         * Deserializes the supplied serialized nodes to the node dispatcher.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @param pNodes Specifies the nodes.
         * @return Returns a reference to the instance.
         */
        Deserialize(pAfterwards: F.Stack, ...pNodes: INode[]): this;
    }
}

declare module "@tripetto/map/lib/branches/culling" {
    export type TBranchCulling = "first" | "each";
}

declare module "@tripetto/map/lib/branches/terminator" {
    export type TBranchTerminator = "continuous" | "abort" | "jump";
}

declare module "@tripetto/map/lib/definition" {
    /** Dependencies */
    import { ISerialize } from "@marblecore/serializer";
    export interface IDefinition extends ISerialize {}
}

declare module "@tripetto/map/lib/conditions/blocks/decorator" {
    /** Dependencies */
    import { TConditionBlockContext } from "@tripetto/map/lib/conditions/blocks/context";
    import { TConditionBlock } from "@tripetto/map/lib/conditions/blocks/type";
    /**
     * Registers a condition block.
     * @param sType Specifies the block type name.
     * @param sName Specifies the localized name for the block.
     * @param pContext Specifies the block context.
     * @param sVersion Specifies the version of the block in SemVer format (defaults to `0.0.0`).
     */
    export function condition(
        sType: string,
        sName: string,
        pContext: TConditionBlockContext,
        sVersion?: string
    ): (pConditionBlock: TConditionBlock) => void;
}

declare module "@tripetto/map/lib/decorators/definition" {
    export function definition<T>(
        sMode: "r" | "w" | "rw"
    ): (
        pTarget: T,
        sProperty: string,
        pPropertyDescriptor?: PropertyDescriptor
    ) => void;
    export function definition<T>(
        pTarget: T,
        sProperty: string,
        pPropertyDescriptor?: PropertyDescriptor
    ): void;
}

declare module "@tripetto/map/lib/decorators/name" {
    import { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    /**
     * Marks a property as a name influencer.
     * @param pTarget Reference to the target.
     * @param sProperty Contains the property.
     * @param pDescriptor Contains the property descriptor.
     */
    export function name<T extends NodeBlock | ConditionBlock>(
        pTarget: T,
        sProperty: string,
        pDescriptor?: PropertyDescriptor
    ): void;
}

declare module "@tripetto/map/lib/nodes/blocks/decorator" {
    import { TNodeBlock } from "@tripetto/map/lib/nodes/blocks/type";
    /**
     * Registers a node block.
     * @param sType Specifies the block type name.
     * @param sName Specifies the localized name for the block.
     * @param sVersion Specifies the version of the block in SemVer format (defaults to `0.0.0`).
     * @param sKind Specifies the block kind. Can be one of the following values:
     * - `headless`: Specifies the block is headless (has no user interface);
     * - `ui`: Specifies the block implements a user interface.
     */
    export function node(
        sType: string,
        sName: string,
        sVersion?: string,
        sKind?: "headless" | "ui"
    ): (pBlock: TNodeBlock) => void;
}

declare module "@tripetto/map/lib/branches/events" {
    /** Dependencies */
    import { IHookPayload } from "@marblecore/hookup";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { TBranchCulling } from "@tripetto/map/lib/branches/culling";
    import { TBranchTerminator } from "@tripetto/map/lib/branches/terminator";
    export type TBranchEvents =
        | "OnCullingChange"
        | "OnTerminatorChange"
        | "OnNameChange";
    export interface IBranchCullingEvent
        extends IHookPayload<"OnCullingChange"> {
        Branch: Branch;
        Culling: TBranchCulling;
    }
    export interface IBranchTerminatorEvent
        extends IHookPayload<"OnTerminatorChange"> {
        Branch: Branch;
        Terminator: TBranchTerminator;
        Jump: Cluster | undefined;
    }
}

declare module "@tripetto/map/lib/conditions/events" {
    /** Dependencies */
    import { IHookPayload } from "@marblecore/hookup";
    import { Condition } from "@tripetto/map/lib/conditions/condition";
    import { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    export type TConditionEvents =
        | "OnBlockAssign"
        | "OnBlockUnassign"
        | "OnBlockChange"
        | "OnNameChange"
        | "OnDisable";
    export interface IConditionBlockAssignEvent
        extends IHookPayload<"OnBlockAssign"> {
        Condition: Condition;
        Block: ConditionBlock;
    }
    export interface IConditionBlockUnassignEvent
        extends IHookPayload<"OnBlockUnassign"> {
        Condition: Condition;
        Block: ConditionBlock;
    }
    export interface IConditionBlockChangeEvent
        extends IHookPayload<"OnBlockChange"> {
        Condition: Condition;
        Block: ConditionBlock | undefined;
    }
}

declare module "@tripetto/map/lib/nodes/events" {
    /** Dependencies */
    import { IHookPayload } from "@marblecore/hookup";
    import { Node } from "@tripetto/map/lib/nodes/node";
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    export type TNodeEvents =
        | "OnBlockAssign"
        | "OnBlockUnassign"
        | "OnBlockChange"
        | "OnTypeChange"
        | "OnLabelChange"
        | "OnDisable";
    export interface INodeBlockAssignEvent
        extends IHookPayload<"OnBlockAssign"> {
        Node: Node;
        Block: NodeBlock;
    }
    export interface INodeBlockUnassignEvent
        extends IHookPayload<"OnBlockUnassign"> {
        Node: Node;
        Block: NodeBlock;
    }
    export interface INodeBlockChangeEvent
        extends IHookPayload<"OnBlockChange"> {
        Node: Node;
        Block: NodeBlock | undefined;
    }
}

declare module "@tripetto/map/lib/branches/constructor" {
    /** Dependencies */
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { Branches } from "@tripetto/map/lib/branches/branches";
    export interface IBranchConstructor {
        new (pBranches: Branches): Branch;
    }
}

declare module "@tripetto/map/lib/clusters/events" {
    export type TClusterEvents = "OnNameChange";
}

declare module "@tripetto/map/lib/clusters/constructor" {
    /** Dependencies */
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { Clusters } from "@tripetto/map/lib/clusters/clusters";
    export interface IClusterConstructor {
        new (pClusters: Clusters): Cluster;
    }
}

declare module "@tripetto/map/lib/conditions/blocks/context" {
    /** Dependencies */
    import { TNodeBlock } from "@tripetto/map/lib/nodes/blocks/type";
    export type TConditionBlockContext =
        | "map"
        | "cluster"
        | "branch"
        | "node"
        | TNodeBlock;
}

declare module "@tripetto/map/lib/conditions/constructor" {
    /** Dependencies */
    import { Condition } from "@tripetto/map/lib/conditions/condition";
    import { Conditions } from "@tripetto/map/lib/conditions/conditions";
    export interface IConditionConstructor {
        new (pConditions: Conditions): Condition;
    }
}

declare module "@tripetto/map/lib/nodes/constructor" {
    /** Dependencies */
    import { Node } from "@tripetto/map/lib/nodes/node";
    import { Nodes } from "@tripetto/map/lib/nodes/nodes";
    export interface INodeConstructor {
        new (pNodes: Nodes): Node;
    }
}
