/*! Tripetto Collector 0.4.23 - Copyright (C) 2018 Tripetto B.V. - All Rights Reserved */

/// <reference path="marblecore.matrix.d.ts" />
/// <reference path="marblecore.serializer.d.ts" />
/// <reference path="marblecore.hookup.d.ts" />

declare module "@tripetto/slots" {
    /** Explicit exports */
    export { Slots } from "@tripetto/slots/lib/slots";
    export { Slot } from "@tripetto/slots/lib/slot";
    export { ISlot } from "@tripetto/slots/lib/interface";
    /** Default export */
    export { Slots as default } from "@tripetto/slots/lib/slots";
    /** Types */
    export { ISlotType } from "@tripetto/slots/lib/type";
    export { Kinds } from "@tripetto/slots/lib/kinds";
    export {
        Transformations
    } from "@tripetto/slots/lib/types/text/transformations";
    /** Events */
    export {
        ISlotPropertyEvent,
        ISlotLabelEvent,
        ISlotRequiredEvent
    } from "@tripetto/slots/lib/events";
    /** Decorators */
    export { register, property } from "@tripetto/slots/lib/decorators";
    /** Built-in types */
    export { BooleanSlot } from "@tripetto/slots/lib/types/boolean/boolean";
    export { NumberSlot } from "@tripetto/slots/lib/types/number/number";
    export { StringSlot } from "@tripetto/slots/lib/types/string/string";
    export { NumericSlot } from "@tripetto/slots/lib/types/numeric/numeric";
    export { TextSlot } from "@tripetto/slots/lib/types/text/text";
    export { DateSlot } from "@tripetto/slots/lib/types/date/date";
}

declare module "@tripetto/slots/lib/slots" {
    import { IDimensionChangeEvent, Matrix } from "@marblecore/matrix";
    import { Slot } from "@tripetto/slots/lib/slot";
    import { ISlotType } from "@tripetto/slots/lib/type";
    import { ISlot } from "@tripetto/slots/lib/interface";
    import { TSlotEvents } from "@tripetto/slots/lib/events";
    import { Kinds } from "@tripetto/slots/lib/kinds";
    export class Slots extends Matrix<Slot, TSlotEvents> {
        /**
         * Registers a slot type.
         * @param pType Specifies the slot instance constructor.
         * @param sType Specifies the slot type.
         */
        static Register<T extends Slot>(
            pType: ISlotType<T>,
            sType: string
        ): void;
        /**
         * Retrieves the slot constructor of the specified type.
         * @param sType Specifies the slot type.
         * @return Returns the slot constructor or `undefined` if the slot type is not
         * available.
         */
        static OfType<T extends Slot>(sType: string): ISlotType<T> | undefined;
        /** Constructs a new slots instance. */
        constructor();
        /** Retrieves if the slots are being deserialized. */
        readonly IsDeserializing: boolean;
        /**
         * Fired when the amount of slots is changed.
         * @param pEvent Specifies the event payload.
         * @event
         */
        protected OnDimensionChange(pEvent: IDimensionChangeEvent<this>): void;
        /**
         * Creates a new meta slot.
         * @param pType Specifies the slot type.
         * @param sReference Specifies the reference.
         * @param nSequence Specifies a sequence number.
         * @return Returns a reference to the slot.
         */
        Meta<T extends Slot>(
            pType: ISlotType<T>,
            sReference: string,
            nSequence?: number
        ): T;
        /**
         * Creates a new static slot.
         * @param pType Specifies the slot type.
         * @param sReference Specifies the reference.
         * @param nSequence Specifies the sequence number.
         * @return Returns a reference to the slot.
         */
        Static<T extends Slot>(
            pType: ISlotType<T>,
            sReference: string,
            nSequence?: number
        ): T;
        /**
         * Creates a new dynamic slot.
         * @param pType Specifies the slot type.
         * @param sReference Specifies the reference.
         * @param nSequence Specifies the sequence number.
         * @return Returns a reference to the slot.
         */
        Dynamic<T extends Slot>(
            pType: ISlotType<T>,
            sReference: string,
            nSequence?: number
        ): T;
        /**
         * Creates a new feature slot.
         * @param pType Specifies the slot type.
         * @param sReference Specifies the reference.
         * @param nSequence Specifies the sequence number.
         * @return Returns a reference to the slot.
         */
        Feature<T extends Slot>(
            pType: ISlotType<T>,
            sReference: string,
            nSequence?: number
        ): T;
        /**
         * Selects a slot.
         * @param sReferenceOrHash Reference or hash of the slot.
         * @param sKind Specifies the optional kind of the slot.
         * @return Returns the slot or `undefined` if no slot is found.
         */
        Select<T extends Slot>(
            sReferenceOrHash: string,
            sKind?: Kinds
        ): T | undefined;
        /**
         * Selects or creates a slot.
         * @param sKind Specifies the kind of the slot.
         * @param pType Specifies the slot type.
         * @param sReference Specifies the reference.
         * @param nSequence Specifies a sequence number.
         * @return Returns the slot or `undefined` if no slot is found.
         */
        SelectOrCreate<T extends Slot>(
            sKind: Kinds,
            pType: ISlotType<T>,
            sReference: string,
            nSequence?: number
        ): T;
        /**
         * Deprecates a slot.
         * @param sReferenceOrHash Reference or hash of the slot.
         * @param sKind Specifies the optional kind of the slot.
         * @return Returns the slot or `undefined` if no slot is found.
         */
        Deprecate<T extends Slot>(
            sReferenceOrHash: string,
            sKind?: Kinds
        ): T | undefined;
        /**
         * Deletes a slot.
         * @param sReferenceOrHash Reference or hash of the slot.
         * @param sKind Specifies the optional kind of the slot.
         * @return Returns the slot or `undefined` if no slot is found.
         */
        Delete<T extends Slot>(
            sReferenceOrHash: string,
            sKind?: Kinds
        ): T | undefined;
        /**
         * Removes all the slots from the collection except the supplied exclude slots.
         * @param sKind Specifies the kind of slots to remove.
         * @param pExclude Specifies the slots to exclude. Those are kept in the slots collection.
         */
        Clean(sKind: Kinds | "*", ...pExclude: (Slot | undefined)[]): void;
        /**
         * Sorts the slots.
         * @return Returns `true` if the order of the slots is changed.
         */
        Sort(): boolean;
        /**
         * Restores the supplied slot.
         * @param pSlot Reference to the slot.
         * @return Returns a reference to the slot.
         */
        Restore<T extends Slot>(pSlot: T): T;
        /**
         * Serializes the slots to an array.
         * @return Returns the slots array.
         */
        Serialize(): ISlot[];
        /**
         * Deserializes slots.
         * @param pSlots Specifies the slots array.
         * @return Returns a reference to the instance.
         */
        Deserialize(pSlots: ISlot[]): this;
        /** Destroys the slots instance. */
        Destroy(): void;
    }
}

declare module "@tripetto/slots/lib/slot" {
    import { Item } from "@marblecore/matrix";
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slots } from "@tripetto/slots/lib/slots";
    import { TSlotEvents } from "@tripetto/slots/lib/events";
    import { ISlot } from "@tripetto/slots/lib/interface";
    import { Kinds } from "@tripetto/slots/lib/kinds";
    export abstract class Slot<
        T extends TSerializeTypes = TSerializeTypes
    > extends Item<TSlotEvents> {
        /** Contains the static type name. */
        static readonly Type: string;
        /** Contains the name of the slot. */
        Name: string;
        /** Contains the placeholder of the slot. */
        Placeholder: string;
        /** Contains an alias for the slot. */
        Alias: string;
        /** Contains if this slot is required. */
        Required: boolean | undefined;
        /** Contains the default slot data value. */
        Default: T | undefined;
        /**
         * Creates a new slot instance.
         * @param sType Specifies the type.
         * @param sKind Specifies the kind.
         * @param sReference Specifies the reference for the slot.
         * @param nSequence Specifies the sequence number.
         */
        constructor(
            sType: string,
            sKind: Kinds,
            sReference: string,
            nSequence: number
        );
        /** Retrieves a reference to the slots. */
        readonly Slots: Slots | undefined;
        /** Retrieves the slot type. */
        readonly Type: string;
        /** Retrieves the slot kind. */
        readonly Kind: Kinds;
        /** Sets the sequence number. */
        Sequence: number;
        /** Retrieves the slot reference. */
        readonly Reference: string;
        /** Retrieves the label for the slot. */
        readonly Label: string;
        /**
         * Emits the `OnSlotChange` event.
         * @param sProperty Specifies the changed property.
         * @event
         */
        PropertyChange(sProperty: string): void;
        /**
         * Serializes a slot.
         * @return Returns a reference to the serialized slot.
         */
        Serialize(): ISlot<T>;
        /**
         * Deserializes to a slot instance.
         * @param pSlot Contains the serialized slot.
         * @return Returns a reference to the instance.
         */
        Deserialize(pSlot: ISlot<T>): this;
        /**
         * Deprecates a slot.
         * @return Returns a reference to the instance.
         */
        Deprecate(): this;
        /**
         * Deletes a slot.
         * @return Returns a reference to the instance.
         */
        Delete(): this;
        /**
         * Converts the supplied data to a valid value.
         * @param pData Specifies the data.
         * @return Returns the data value.
         */
        abstract ToValue(pData: TSerializeTypes): T;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        abstract ToString(pData: TSerializeTypes): string;
    }
}

declare module "@tripetto/slots/lib/interface" {
    /** Dependencies */
    import { ISerialize, TSerializeTypes } from "@marblecore/serializer";
    import { Kinds } from "@tripetto/slots/lib/kinds";
    export interface ISlot<T extends TSerializeTypes = TSerializeTypes>
        extends ISerialize {
        /** Contains the id for the serialized slot. */
        readonly Id: string;
        /** Contains the slot type. */
        readonly Type: string;
        /** Contains the slot kind. */
        readonly Kind: Kinds;
        /** Contains the slot referemce. */
        readonly Reference: string;
        /** Contains the slot sequence number. */
        readonly Sequence: number;
        /** Contains the slot name. */
        readonly Name: string;
        /** Contains the slot placeholder. */
        readonly Placeholder: string;
        /** Contains the slot alias. */
        readonly Alias: string;
        /** Contains if the slot is required. */
        readonly Required: boolean | undefined;
        /** Contains the default slot value. */
        readonly Default: T | undefined;
    }
}

declare module "@tripetto/slots/lib/type" {
    /** Dependencies */
    import { Slot } from "@tripetto/slots/lib/slot";
    import { Kinds } from "@tripetto/slots/lib/kinds";
    export interface ISlotType<T extends Slot = Slot> {
        /** Contains the slot type. */
        readonly Type: string;
        /**
         * Creates a new slot instance.
         * @param sType Specifies the type.
         * @param sKind Specifies the kind.
         * @param sReference Specifies the reference.
         * @param nSequence Specifies the sequence number.
         */
        new (
            sType: string,
            sKind: Kinds,
            sReference: string,
            nSequence: number
        ): T;
    }
}

declare module "@tripetto/slots/lib/kinds" {
    /** Specifies the slot kinds. */
    export type Kinds = "meta" | "static" | "dynamic" | "feature";
    /**
     * Retrieves the static order index for the supplied kind.
     * @param sKind Specifies the kind.
     * @return Returns the order index.
     */
    export function KindOrderIndexFor(sKind: Kinds): number;
}

declare module "@tripetto/slots/lib/types/text/transformations" {
    export type Transformations =
        | "none"
        | "capitalize"
        | "capitalize-words"
        | "capitalize-sentences"
        | "uppercase"
        | "lowercase";
}

declare module "@tripetto/slots/lib/events" {
    /** Dependencies */
    import { IHookPayload, THooks } from "@marblecore/hookup";
    import { Slot } from "@tripetto/slots/lib/slot";
    export type TSlotEvents =
        | "OnSlotLabel"
        | "OnSlotRequired"
        | "OnSlotProperty";
    export interface ISlotEvent<H extends THooks, T extends Slot>
        extends IHookPayload<H> {
        /** Reference to the slot. */
        readonly Slot: T;
    }
    export interface ISlotLabelEvent<T extends Slot>
        extends ISlotEvent<"OnSlotLabel", T> {}
    export interface ISlotRequiredEvent<T extends Slot>
        extends ISlotEvent<"OnSlotRequired", T> {}
    export interface ISlotPropertyEvent<T extends Slot>
        extends ISlotEvent<"OnSlotProperty", T> {
        /** Specifies the changed property. */
        readonly Property: string;
    }
}

declare module "@tripetto/slots/lib/decorators" {
    import { Slot } from "@tripetto/slots/lib/slot";
    import { ISlotType } from "@tripetto/slots/lib/type";
    /**
     * Registers a slot type.
     * @param sType Specifies the slot type name.
     */
    export function register<T extends Slot>(
        sType: string
    ): (pType: ISlotType<T>) => void;
    /**
     * Marks a property as a slot property.
     * @param pTarget Reference to the slot.
     * @param sProperty Specifies the slot property.
     * @param pDescriptor Reference to the property descriptor.
     */
    export function property(
        pTarget: Slot,
        sProperty: string,
        pDescriptor?: PropertyDescriptor
    ): void;
}

declare module "@tripetto/slots/lib/types/boolean/boolean" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots/lib/slot";
    export class BooleanSlot extends Slot<boolean> {
        /**
         * Converts the supplied data to a valid boolean value.
         * @param pData Specifies the data.
         * @return Returns the boolean value.
         */
        ToValue(pData: TSerializeTypes): boolean;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        ToString(pData: TSerializeTypes): string;
    }
}

declare module "@tripetto/slots/lib/types/number/number" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots/lib/slot";
    export class NumberSlot extends Slot<number> {
        /**
         * Converts the supplied data to a valid number value.
         * @param pData Specifies the data.
         * @return Returns the number value.
         */
        ToValue(pData: TSerializeTypes): number;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        ToString(pData: TSerializeTypes): string;
    }
}

declare module "@tripetto/slots/lib/types/string/string" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots/lib/slot";
    export class StringSlot extends Slot<string> {
        /**
         * Converts the supplied data to a valid string value.
         * @param pData Specifies the data.
         * @return Returns the string value.
         */
        ToValue(pData: TSerializeTypes): string;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        ToString(pData: TSerializeTypes): string;
    }
}

declare module "@tripetto/slots/lib/types/numeric/numeric" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { NumberSlot } from "@tripetto/slots/lib/types/number/number";
    export class NumericSlot extends NumberSlot {
        /** Contains the separtor sign. */
        Separator: string;
        /** Contains the decimal sign. */
        Decimal: string;
        /** Contains the minus sign. */
        Minus: string;
        /** Contains the prefix. */
        Prefix: string;
        /** Contains the suffix. */
        Suffix: string;
        /** Sets the numeric precision. */
        Precision: number;
        /**
         * Converts the supplied data to a valid numericvalue.
         * @param pData Specifies the data.
         * @return Returns the numeric value.
         */
        ToValue(pData: TSerializeTypes): number;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        ToString(pData: TSerializeTypes): string;
    }
}

declare module "@tripetto/slots/lib/types/text/text" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { StringSlot } from "@tripetto/slots/lib/types/string/string";
    import { Transformations } from "@tripetto/slots/lib/types/text/transformations";
    export class TextSlot extends StringSlot {
        Transformation: Transformations;
        /**
         * Converts the supplied data to a valid text value.
         * @param pData Specifies the data.
         * @return Returns the text value.
         */
        ToValue(pData: TSerializeTypes): string;
    }
}

declare module "@tripetto/slots/lib/types/date/date" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots/lib/slot";
    export class DateSlot extends Slot<number> {
        /**
         * Converts the supplied data to a date value.
         * @param pData Specifies the data.
         * @return Returns the date value.
         */
        ToValue(pData: TSerializeTypes | Date): number;
        /**
         * Converts the supplied data to a valid date object.
         * @param pData Specifies the data.
         * @return Returns the data as a date object.
         */
        ToDate(pData: TSerializeTypes | Date): Date;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        ToString(pData: TSerializeTypes | Date): string;
    }
}
