/*! Tripetto Collector 0.4.23 - Copyright (C) 2018 Tripetto B.V. - All Rights Reserved */

/// <reference path="marblecore.matrix.d.ts" />
/// <reference path="marblecore.callback.d.ts" />
/// <reference path="tripetto.slots.d.ts" />
/// <reference path="marblecore.serializer.d.ts" />
/// <reference path="marblecore.hookup.d.ts" />
/// <reference path="marblecore.functional.d.ts" />

declare module "@tripetto/vfsm" {
    /** Explicit exports */
    export { Conditions } from "@tripetto/vfsm/lib/conditions/conditions";
    export { Condition } from "@tripetto/vfsm/lib/conditions/condition";
    export { ICondition } from "@tripetto/vfsm/lib/conditions/interface";
    export { Branches } from "@tripetto/vfsm/lib/branches/branches";
    export { Branch } from "@tripetto/vfsm/lib/branches/branch";
    export { IBranch } from "@tripetto/vfsm/lib/branches/interface";
    export { States } from "@tripetto/vfsm/lib/states/states";
    export { State } from "@tripetto/vfsm/lib/states/state";
    export { IState } from "@tripetto/vfsm/lib/states/interface";
    export { Transducers } from "@tripetto/vfsm/lib/transducers/transducers";
    export { Transducer } from "@tripetto/vfsm/lib/transducers/transducer";
    export { ITransducer } from "@tripetto/vfsm/lib/transducers/interface";
    export { Instance } from "@tripetto/vfsm/lib/instance/instance";
    export { Ontology } from "@tripetto/vfsm/lib/ontology";
    export {
        IOntology,
        IOntologyProperties
    } from "@tripetto/vfsm/lib/interface";
    export { ISnapshot } from "@tripetto/vfsm/lib/snapshot/snapshot";
    export { Store } from "@tripetto/vfsm/lib/store/store";
    export { Data } from "@tripetto/vfsm/lib/store/data";
    export {
        IData,
        IDataValue,
        IDataValues
    } from "@tripetto/vfsm/lib/store/interface";
    /** Enumerations */
    export { Errors } from "@tripetto/vfsm/enum/errors";
    /** Decorators */
    export { ontology } from "@tripetto/vfsm/lib/decorator";
    /** Events */
    export {
        IOntologyEndEvent,
        IOntologyInstanceCreateEvent,
        IOntologyInstanceEndEvent,
        IOntologyInstancePauseEvent,
        IOntologyInstanceStartEvent,
        IOntologyInstanceStopEvent,
        IOntologyLoadEvent,
        IOntologyPauseEvent,
        IOntologyStartEvent,
        IOntologyStopEvent,
        IOntologyUnloadEvent
    } from "@tripetto/vfsm/lib/events";
    export {
        IInstanceCreateEvent,
        IInstanceEndEvent,
        IInstancePauseEvent,
        IInstanceStartEvent,
        IInstanceStopEvent
    } from "@tripetto/vfsm/lib/instance/events";
    export {
        IBranchCancelEvent,
        IBranchEnterEvent,
        IBranchLeaveEvent,
        IBranchReadyEvent,
        IBranchSkipEvent
    } from "@tripetto/vfsm/lib/branches/events";
    export {
        IStateCancelEvent,
        IStateEnterEvent,
        IStateLeaveEvent,
        IStateSkipEvent,
        IStateValidatedEvent
    } from "@tripetto/vfsm/lib/states/events";
    export {
        ITransducerCancelEvent,
        ITransducerEnterEvent,
        ITransducerLeaveEvent,
        ITransducerSkipEvent,
        ITransducerValidatedEvent
    } from "@tripetto/vfsm/lib/transducers/events";
    export { IStoreEvent } from "@tripetto/vfsm/lib/store/events";
}

declare module "@tripetto/vfsm/lib/conditions/conditions" {
    /** Dependencies */
    import { StrictMatrix } from "@marblecore/matrix";
    import { Condition } from "@tripetto/vfsm/lib/conditions/condition";
    import { Branch } from "@tripetto/vfsm/lib/branches/branch";
    import { ICondition } from "@tripetto/vfsm/lib/conditions/interface";
    export class Conditions extends StrictMatrix<
        Conditions,
        Condition,
        void,
        ICondition
    > {
        /**
         * Constructs a new condition dispatcher.
         * @param Reference to the parent branch.
         */
        constructor(pBranch: Branch);
        /** Retrieves a reference to the parent branch. */
        readonly Branch: Branch;
        /**
         * Creates a condition.
         * @param pCondition Reference to the condition ontology.
         * @return Returns a reference to the condition instance.
         */
        Create<T extends Condition>(pCondition: ICondition): T;
    }
}

declare module "@tripetto/vfsm/lib/conditions/condition" {
    /** Dependencies */
    import { Callback } from "@marblecore/callback";
    import { Factory } from "@tripetto/vfsm/lib/factory";
    import { Conditions } from "@tripetto/vfsm/lib/conditions/conditions";
    import { Branch } from "@tripetto/vfsm/lib/branches/branch";
    import { State } from "@tripetto/vfsm/lib/states/state";
    import { Instance } from "@tripetto/vfsm/lib/instance/instance";
    import { ICondition } from "@tripetto/vfsm/lib/conditions/interface";
    export abstract class Condition<
        T extends ICondition = ICondition
    > extends Factory<Conditions, T> {
        /**
         * Constructs a new condition.
         * @param pConditions Reference to the condition dispatcher.
         * @param pCondition Reference to the condition data.
         */
        constructor(pConditions: Conditions, pCondition: T);
        /** Retrieves a reference to the conditions. */
        protected readonly Conditions: Conditions;
        /** Retrieves a reference to the parent branch. */
        readonly Branch: Branch;
        readonly State: State;
        /** Retrieves the condition properties. */
        readonly Props: T;
        /**
         * Invoked when the VFSM wants to verify the condition.
         * @param pInstance Reference to the instance.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous condition callback.
         * @return Returns the result of the condition or the callback reference if
         * the condition should be handled asynchronously.
         * @event
         */
        abstract OnCondition(
            pInstance: Instance,
            pCallback: Callback<boolean>
        ): boolean | Callback<boolean>;
    }
}

declare module "@tripetto/vfsm/lib/conditions/interface" {
    /** Dependencies */
    import { IOntologyProperties } from "@tripetto/vfsm/lib/interface";
    export interface ICondition extends IOntologyProperties {}
}

declare module "@tripetto/vfsm/lib/branches/branches" {
    /** Dependencies */
    import { StrictMatrix } from "@marblecore/matrix";
    import { Branch } from "@tripetto/vfsm/lib/branches/branch";
    import { State } from "@tripetto/vfsm/lib/states/state";
    import { IBranch } from "@tripetto/vfsm/lib/branches/interface";
    export class Branches extends StrictMatrix<
        Branches,
        Branch,
        void,
        IBranch
    > {
        /**
         * Constructs a new branch dispatcher.
         * @param pState Reference to the parent state.
         */
        constructor(pState: State);
        /** Retrieves a reference to the parent state. */
        readonly State: State;
        /**
         * Creates a branch.
         * @param pBranch Reference to the branch ontology.
         * @param pAncestor Reference to the ancestor state.
         * @return Returns a reference to the branch instance.
         */
        Create<T extends Branch>(pBranch: IBranch, pAncestor: State): T;
    }
}

declare module "@tripetto/vfsm/lib/branches/branch" {
    import { Callback } from "@marblecore/callback";
    import { Factory } from "@tripetto/vfsm/lib/factory";
    import { Conditions } from "@tripetto/vfsm/lib/conditions/conditions";
    import { Branches } from "@tripetto/vfsm/lib/branches/branches";
    import { State } from "@tripetto/vfsm/lib/states/state";
    import { Instance } from "@tripetto/vfsm/lib/instance/instance";
    import { IBranch } from "@tripetto/vfsm/lib/branches/interface";
    import { TBranchCulling } from "@tripetto/vfsm/lib/branches/culling";
    import { TBranchEvents } from "@tripetto/vfsm/lib/branches/events";
    export abstract class Branch<T extends IBranch = IBranch> extends Factory<
        Branches,
        T,
        TBranchEvents
    > {
        /**
         * Constructs a new branch.
         * @param pBranches Reference to the branch dispatcher.
         * @param pBranch Reference to the branch data.
         */
        constructor(pBranches: Branches, pBranch: T);
        /** Retrieves a reference to the branches. */
        protected readonly Branches: Branches;
        /** Retrieves an array with states. */
        readonly States: State[];
        readonly Entry: State | undefined;
        /** Retrieves a reference to the parent state. */
        readonly State: State;
        /** Retrieves a reference to the condition dispatcher. */
        readonly Conditions: Conditions;
        /** Retrieves the branch properties. */
        readonly Props: T;
        /** Retrieves the culling mode for the branch. */
        readonly Culling: TBranchCulling;
        /** Retrieves if this branch should spawn a new stack. */
        readonly Spawn: boolean;
        /** Retrieves if this is the main branch. */
        readonly IsMain: boolean;
        /**
         * Pushes a state to the branch.
         * @param pState Reference to the state to push.
         */
        Push(pState: State): void;
        /**
         * Invoked when the VFSM wants to verify the condition for the branch.
         * @param pInstance Reference to the instance.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous condition callback.
         * @return Returns the result of the condition or the callback reference if
         * the condition should be handled asynchronously.
         * @event
         */
        abstract OnCondition(
            pInstance: Instance,
            pCallback: Callback<boolean>
        ): boolean | Callback<boolean>;
        /**
         * Invoked when the branch is ready for entering. The condition is approved.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnReady(pInstance: Instance): void;
        /**
         * Invoked when the branch is entered.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnEnter(pInstance: Instance): void;
        /**
         * Invoked when the branch is skipped due to a failing enter condition.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnSkip(pInstance: Instance): void;
        /**
         * Invoked when the branch is left.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnLeave(pInstance: Instance): void;
        /**
         * Invoked when the branch is canceled.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnCancel(pInstance: Instance): void;
    }
}

declare module "@tripetto/vfsm/lib/branches/interface" {
    /** Dependencies */
    import { IOntologyProperties } from "@tripetto/vfsm/lib/interface";
    import { TBranchCulling } from "@tripetto/vfsm/lib/branches/culling";
    export interface IBranch extends IOntologyProperties {
        /**
         * Specifies the jump reference for the branch. If this property is omitted
         * or an empty string, the branch successor will be determined
         * automatically. If this property is set to 'abort' the stack pointer will
         * be killed after the branch. If this property is set to the identifier
         * hash of another state, the stack pointer is redirected to that state.
         */
        Jump?: string | "abort";
        /** Specifies the culling mode for the branch. */
        Culling?: TBranchCulling;
        /** Specifies if this branch should spawn a new stack. */
        Spawn?: boolean;
    }
}

declare module "@tripetto/vfsm/lib/states/states" {
    /** Dependencies */
    import { StrictMatrix } from "@marblecore/matrix";
    import { Condition } from "@tripetto/vfsm/lib/conditions/condition";
    import { Branch } from "@tripetto/vfsm/lib/branches/branch";
    import { Transducer } from "@tripetto/vfsm/lib/transducers/transducer";
    import { State } from "@tripetto/vfsm/lib/states/state";
    import { Main } from "@tripetto/vfsm/lib/states/main";
    import { Ontology } from "@tripetto/vfsm/lib/ontology";
    import { IState } from "@tripetto/vfsm/lib/states/interface";
    export class States extends StrictMatrix<States, State, void, IState> {
        /**
         * Constructs a new state dispatcher.
         * @param pOntology Reference to the ontology.
         */
        constructor(pOntology: Ontology<State, Transducer, Branch, Condition>);
        /** Retrieves a reference to the ontology. */
        readonly Ontology: Ontology<State, Transducer, Branch, Condition>;
        /** Retrieves the state level information of the last created state. */
        readonly Levels: number[];
        /** Retrieves the numerator of the last created state. */
        readonly Numerator: number;
        /**
         * Creates a new main state.
         * @return Returns the main state.
         */
        Main(): Main;
        /**
         * Creates a state.
         * @param pState Reference to the state data.
         * @param pBranch Reference to the parent branch.
         * @param pAncestor Reference to the ancestor state.
         * @param nStates Specifies the number of states.
         * @param nNumerator Specifies the numerator of the state.
         * @return Returns a reference to the state.
         */
        Create<T extends State>(
            pState: IState,
            pBranch: Branch,
            pAncestor: State,
            nStates: number,
            nNumerator: number
        ): T;
    }
}

declare module "@tripetto/vfsm/lib/states/state" {
    /** Dependencies */
    import { Await, Callback } from "@marblecore/callback";
    import { Factory } from "@tripetto/vfsm/lib/factory";
    import { Condition } from "@tripetto/vfsm/lib/conditions/condition";
    import { Branches } from "@tripetto/vfsm/lib/branches/branches";
    import { Branch } from "@tripetto/vfsm/lib/branches/branch";
    import { Transducers } from "@tripetto/vfsm/lib/transducers/transducers";
    import { Transducer } from "@tripetto/vfsm/lib/transducers/transducer";
    import { States } from "@tripetto/vfsm/lib/states/states";
    import { Instance } from "@tripetto/vfsm/lib/instance/instance";
    import { Ontology } from "@tripetto/vfsm/lib/ontology";
    import { IState } from "@tripetto/vfsm/lib/states/interface";
    import { TStateEvents } from "@tripetto/vfsm/lib/states/events";
    export abstract class State<T extends IState = IState> extends Factory<
        States,
        T,
        TStateEvents
    > {
        /**
         * Constructs a new state.
         * @param pStates Reference to the state dispatcher.
         * @param pState Specifies the state data.
         */
        constructor(pStates: States, pState: T);
        /** Retrieves a reference to the states. */
        protected readonly States: States;
        /** Retrieves a reference to the ontology. */
        readonly Ontology: Ontology<State, Transducer, Branch, Condition>;
        /** Retrieves a reference to the transducer dispatcher. */
        readonly Transducers: Transducers;
        /** Retrieves a reference to the branch dispatcher. */
        readonly Branches: Branches;
        /** Retrieves if the state is the main state. */
        readonly IsMain: boolean;
        /** Retrieves the state properties. */
        readonly Props: T;
        /**
         * Retrieves the state levels. Each level is represented as a new array item
         * with the index number of the level.
         */
        readonly Levels: number[];
        /** Returns the numerator for the state. */
        readonly Numerator: number;
        /** Retrieves the total number of the child states. */
        readonly StateCount: number;
        /** Retrieves the total number of transducers. */
        readonly TransducerCount: number;
        /**
         * Sets the state metrics based on the supplied loader data.
         * @param nStates Specifies the number of states.
         * @param nTransducers Specifies the number of transducers.
         */
        Metrics(nStates: number, nTransducers: number): void;
        /**
         * Invoked when the VFSM wants to verify the enter condition for the state.
         * This event is invoked just before the state is entered and it is also
         * invoked for the main state. This allows you to use a condition for the
         * main state which lets you control the main application branch. If you
         * override this function you have to make sure to return an appropriate
         * value for the main state.
         * @param pInstance Reference to the instance.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous condition callback.
         * @return Returns the result of the condition or the callback reference if
         * the condition should be handled asynchronously.
         * @event
         */
        abstract OnEnterCondition(
            pInstance: Instance,
            pCallback: Callback<boolean>
        ): boolean | Callback<boolean>;
        /**
         * Invoked when state is entered.
         * @param pInstance Reference to the instance.
         * @param pAwait Reference to the await pointer. Pass this pointer as return
         * value to enable the asynchronous callback.
         * @return Returns the callback reference if the function should be handled
         * asynchronously.
         * @event
         */
        abstract OnEnter(pInstance: Instance, pAwait: Await): void | Await;
        /**
         * Invoked when state is skipped due to a failing enter condition.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnSkip(pInstance: Instance): void;
        /**
         * Invoked when the VFSM wants to verify the leave condition for the state.
         * This event is invoked just before the state is left.
         * @param pInstance Reference to the instance.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous condition callback.
         * @return Returns the result of the condition or the callback reference if
         * the condition should be handled asynchronously.
         * @event
         */
        abstract OnLeaveCondition(
            pInstance: Instance,
            pCallback: Callback<boolean>
        ): boolean | Callback<boolean>;
        /**
         * Invoked when the state is left.
         * @param pInstance Reference to the instance.
         * @return Returns the callback reference if the function should be handled
         * asynchronously.
         * @event
         */
        OnLeave(pInstance: Instance): void;
        /**
         * Invoked when the state cannot be left due to a failing leave condition.
         * @param pInstance Reference to the instance.
         * @param pAwait Reference to the await pointer. Pass this pointer as return
         * value to enable the asynchronous callback.
         * @event
         */
        OnLeaveError(pInstance: Instance, pAwait: Await): void | Await;
        /**
         * Invoked when the state is canceled.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnCancel(pInstance: Instance): void;
        /**
         * Invoked when the state needs to be validated.
         * @param pInstance Reference to the instance.
         * @param sCurrent Specifies the current validation state.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous validation callback.
         * @return Returns the result of the validation or the callback reference if
         * the validation should be handled asynchronously.
         * @event
         */
        abstract OnValidate(
            pInstance: Instance,
            sCurrent: "unknown" | "fail" | "pass",
            pCallback: Callback<boolean>
        ): boolean | Callback<boolean>;
        /**
         * Invoked when the state is validated.
         * @param sResult Specifies the validation state.
         * @param sType Specifies if the validation type.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnValidated(
            sResult: "fail" | "pass",
            sType: "initial" | "revalidate",
            pInstance: Instance
        ): void;
    }
}

declare module "@tripetto/vfsm/lib/states/interface" {
    /** Dependencies */
    import { IOntologyProperties } from "@tripetto/vfsm/lib/interface";
    export interface IState extends IOntologyProperties {}
}

declare module "@tripetto/vfsm/lib/transducers/transducers" {
    /** Dependencies */
    import { StrictMatrix } from "@marblecore/matrix";
    import { Transducer } from "@tripetto/vfsm/lib/transducers/transducer";
    import { State } from "@tripetto/vfsm/lib/states/state";
    import { ITransducer } from "@tripetto/vfsm/lib/transducers/interface";
    export class Transducers extends StrictMatrix<
        Transducers,
        Transducer,
        void,
        ITransducer
    > {
        /**
         * Constructs a new transducer dispatcher.
         * @param pState Reference to the state.
         */
        constructor(pState: State);
        /** Retrieves a reference to the parent state. */
        readonly State: State;
        /**
         * Creates a transducer.
         * @param pTransducer Reference to the transducer ontology.
         * @return Returns a reference to the transducer instance.
         */
        Create<T extends Transducer>(pTransducer: ITransducer): T;
    }
}

declare module "@tripetto/vfsm/lib/transducers/transducer" {
    /** Dependencies */
    import { Await, Callback } from "@marblecore/callback";
    import { Slots } from "@tripetto/slots";
    import { Factory } from "@tripetto/vfsm/lib/factory";
    import { State } from "@tripetto/vfsm/lib/states/state";
    import { Transducers } from "@tripetto/vfsm/lib/transducers/transducers";
    import { Instance } from "@tripetto/vfsm/lib/instance/instance";
    import { ITransducer } from "@tripetto/vfsm/lib/transducers/interface";
    import { TTransducerEvents } from "@tripetto/vfsm/lib/transducers/events";
    export abstract class Transducer<
        T extends ITransducer = ITransducer
    > extends Factory<Transducers, T, TTransducerEvents> {
        /**
         * Constructs a new transducer.
         * @param pTransducers Reference to the transducer dispatcher.
         * @param pTransducer Reference to the transducer data.
         */
        constructor(pTransducers: Transducers, pTransducer: T);
        /** Retrieves a reference to the transducers. */
        protected readonly Transducers: Transducers;
        /** Retrieves a reference to the parent state. */
        readonly State: State;
        /** Retrieves a reference to the slots. */
        readonly Slots: Slots;
        /** Retrieves the transducer properties. */
        readonly Props: T;
        /**
         * Invoked when the VFSM wants to verify the condition for the transducer.
         * This event is invoked just before the transducer is entered.
         * @param pInstance Reference to the instance.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous condition callback.
         * @return Returns the result of the condition or the callback reference if
         * the condition should be handled asynchronously.
         * @event
         */
        abstract OnEnterCondition(
            pInstance: Instance,
            pCallback: Callback<boolean>
        ): boolean | Callback<boolean>;
        /**
         * Invoked when the transducer is entered.
         * @param pInstance Reference to the instance.
         * @param pAwait Reference to the await pointer. Pass this pointer as return
         * value to enable asynchronous validation.
         * @return Return the await pointer to implement asynchronous transducer validation.
         * @event
         */
        abstract OnEnter(pInstance: Instance, pAwait: Await): void | Await;
        /**
         * Invoked when the transducer is skipped due to a failing enter condition.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnSkip(pInstance: Instance): void;
        /**
         * Invoked when the transducer is left.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnLeave(pInstance: Instance): void;
        /**
         * Invoked when the transducer is canceled.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnCancel(pInstance: Instance): void;
        /**
         * Invoked when the transducer is validated.
         * @param pInstance Reference to the instance.
         * @param sCurrent Specifies the current transducer validation state.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous validation callback.
         * @param bObserved Specifies if the transducer is observed.
         * @return Returns the result of the validation or the callback reference if
         * the validation should be handled asynchronously.
         * @event
         */
        abstract OnValidate(
            pInstance: Instance,
            sCurrent: "unknown" | "fail" | "pass",
            pCallback: Callback<boolean>,
            bObserved: boolean
        ): boolean | Callback<boolean>;
        /**
         * Invoked when the transducer is validated.
         * @param sResult Specifies the transducer validation state.
         * @param sType Specifies if the validation type.
         * @param bObserved Specifies if the transducer is observed.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnValidated(
            sResult: "fail" | "pass",
            sType: "initial" | "revalidate",
            bObserved: boolean,
            pInstance: Instance
        ): void;
    }
}

declare module "@tripetto/vfsm/lib/transducers/interface" {
    /** Dependencies */
    import { ISlot } from "@tripetto/slots";
    import { IOntologyProperties } from "@tripetto/vfsm/lib/interface";
    export interface ITransducer extends IOntologyProperties {
        /** Contains the slots for the transducer. */
        Slots?: ISlot[];
    }
}

declare module "@tripetto/vfsm/lib/instance/instance" {
    import { Item } from "@marblecore/matrix";
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots";
    import { Store } from "@tripetto/vfsm/lib/store/store";
    import { StoreItem } from "@tripetto/vfsm/lib/store/item";
    import { Data } from "@tripetto/vfsm/lib/store/data";
    import { IData } from "@tripetto/vfsm/lib/store/interface";
    import { Stack } from "@tripetto/vfsm/lib/stack/stack";
    import { Condition } from "@tripetto/vfsm/lib/conditions/condition";
    import { Branch } from "@tripetto/vfsm/lib/branches/branch";
    import { Transducer } from "@tripetto/vfsm/lib/transducers/transducer";
    import { State } from "@tripetto/vfsm/lib/states/state";
    import { IInstanceSnapshot } from "@tripetto/vfsm/lib/instance/snapshot";
    import { Ontology } from "@tripetto/vfsm/lib/ontology";
    import { TThreading } from "@tripetto/vfsm/lib/threading";
    import { TInstanceEvents } from "@tripetto/vfsm/lib/instance/events";
    export class Instance extends Item<TInstanceEvents> {
        /**
         * Constructs the instance.
         * @param pOntology Reference to the ontology.
         * @param sThreading Specifies if the instance allows single or multi threading.
         * @param bVerbose Specifies if verbose logging is enabled.
         * @param pBootstrap Specifies the bootstrap state.
         * @param pStore Reference to a data store for the instance. If this parameter is `undefined` a new data store will be initialized.
         * @param pResume Reference to resume data. If supplied the instance will be restored using the resume data.
         */
        constructor(
            pOntology: Ontology<State, Transducer, Branch, Condition>,
            sThreading?: TThreading,
            bVerbose?: boolean,
            pBootstrap?: State,
            pStore?: Store,
            pResume?: IInstanceSnapshot
        );
        /** Retrieves a reference to the ontology. */
        readonly Ontology: Ontology<State, Transducer, Branch, Condition>;
        /** Retrieves if the instance allows concurrency. */
        readonly AllowConcurrency: boolean;
        /** Retrieves if the instance is running. */
        readonly IsRunning: boolean;
        /** Retrieves if the instance is stopped (interrupted). */
        readonly IsStopped: boolean;
        /** Retrieves if the instance is paused. */
        readonly IsPaused: boolean;
        /**
         * Retrieves if the instance is at the end. It is best to use this property
         * in an asynchronous call. When called synchronous the value
         * may be wrong in some cases.
         */
        readonly IsAtEnd: boolean;
        /** Retrieves if verbose logging is enabled. */
        readonly IsVerbose: boolean;
        /** Retrieves a reference to the stack pointer. */
        readonly Pointer: Stack | undefined;
        /** Retrieves a unique key for the current instance state. */
        readonly Key: string;
        /** Retrieves the slot data. */
        readonly Store: Store;
        /** Retrieves the slots. */
        readonly Slots: Slot[];
        /** Retrieves the value store. */
        readonly Values: IData;
        /** Retrieves the number of steps in this instance. */
        readonly Steps: number;
        /** Retrieves error information. */
        readonly Errors: number;
        /**
         * Assures a stack is bind to the pointer.
         * @param pStack Reference to the stack.
         * @return Returns the pointer.
         */
        AssurePointer(pStack: Stack): Stack | undefined;
        /** Releases the current pointer. */
        ReleasePointer(): void;
        /** Pushes the current stack to the stack trace. */
        Push(): void;
        /**
         * Executes the supplied function within the supplied stack context.
         * @param pStack Specifies the stack context.
         * @param fnExecute Specifies the function to execute.
         */
        ExecuteWithinContext<T>(pStack: Stack, fnExecute: () => T): T;
        /**
         * Spawns a new instance.
         * @param bSynchronous Specifies if the instance should be single or multi theading.
         * @param bVerbose Specifies if verbose logging is enabled.
         * @param pState Specifies the bootstrap state.
         * @return Returns a reference to the new instance.
         */
        Spawn(
            sThreading: TThreading,
            bVerbose: boolean,
            pState: State
        ): Instance;
        /**
         * Starts the instance.
         * @return Returns `true` if the instance is started.
         */
        Start(): boolean;
        /**
         * Stops the instance.
         * @return Returns `true` if the instance was running and is stopped.
         */
        Stop(): boolean;
        /**
         * Pauses the instance.
         * @return Returns the instance snapshot which is necessary to resume the
         * instance.
         */
        Pause(): IInstanceSnapshot;
        /**
         * Shifts a stack trace position.
         * @param nIndex Specifies the index of the stack.
         * @param bContinuous Specifies if the shift is continuous and the stack
         * trace needs to be maintained.
         * @return Returns `true` if the shift is succeeded.
         */
        Shift(nIndex: number, bContinuous: boolean): boolean;
        /**
         * Steps backwards in the stack.
         * @param nSteps Specifies the number of steps to shift.
         * @return Returns `true` if the step succeeded.
         */
        StepBackward(nSteps?: number): boolean;
        /**
         * Retrieve the store for the supplied slot.
         * @param pSlotOrHash Reference to the slot or slot hash.
         * @return Returns the slot data store or `undefined` if no store is found.
         */
        StoreForSlot<T extends TSerializeTypes, S extends Slot<T>>(
            pSlotOrHash: Slot | string
        ): StoreItem<T, S> | undefined;
        /**
         * Retrieves the contextual data store for the supplied slot.
         * @param pSlotOrHash Reference to the slot or slot hash.
         * @param pState Reference to the state.
         * @return Returns the data or `undefined`  if no data store was found.
         */
        DataForSlot<T extends TSerializeTypes>(
            pSlotOrHash: Slot | string,
            pState: State
        ): Data<T> | undefined;
        /**
         * Snapshots the instance.
         * @return Returns the instance snapshot.
         */
        Snapshot(): IInstanceSnapshot;
        /**
         * Logs a message to the log buffer of the VFSM.
         * @param sMessage Specifies the log message.
         */
        LogMessage(sMessage: string): void;
    }
}

declare module "@tripetto/vfsm/lib/ontology" {
    import { Matrix } from "@marblecore/matrix";
    import { Errors } from "@tripetto/vfsm/enum/errors";
    import { Branch } from "@tripetto/vfsm/lib/branches/branch";
    import { IBranch } from "@tripetto/vfsm/lib/branches/interface";
    import { TBranch } from "@tripetto/vfsm/lib/branches/type";
    import { Condition } from "@tripetto/vfsm/lib/conditions/condition";
    import { ICondition } from "@tripetto/vfsm/lib/conditions/interface";
    import { TCondition } from "@tripetto/vfsm/lib/conditions/type";
    import { IConstructors } from "@tripetto/vfsm/lib/constructors";
    import { TOntologyEvents } from "@tripetto/vfsm/lib/events";
    import { Instance } from "@tripetto/vfsm/lib/instance/instance";
    import { IInstanceSnapshot } from "@tripetto/vfsm/lib/instance/snapshot";
    import { IOntology } from "@tripetto/vfsm/lib/interface";
    import { ISnapshot } from "@tripetto/vfsm/lib/snapshot/snapshot";
    import { IState } from "@tripetto/vfsm/lib/states/interface";
    import { Main } from "@tripetto/vfsm/lib/states/main";
    import { State } from "@tripetto/vfsm/lib/states/state";
    import { States } from "@tripetto/vfsm/lib/states/states";
    import { TState } from "@tripetto/vfsm/lib/states/type";
    import { TThreading } from "@tripetto/vfsm/lib/threading";
    import { ITransducer } from "@tripetto/vfsm/lib/transducers/interface";
    import { Transducer } from "@tripetto/vfsm/lib/transducers/transducer";
    import { TTransducer } from "@tripetto/vfsm/lib/transducers/type";
    export abstract class Ontology<
        S extends State,
        T extends Transducer,
        B extends Branch,
        C extends Condition,
        O extends IOntology = IOntology
    > extends Matrix<Instance, TOntologyEvents> {
        /**
         * Constructs a new ontology.
         * @param pState Reference to the state constructor.
         * @param pTransducer Reference to the transducer constructor.
         * @param pBranch Reference to the branch constructor.
         * @param pCondition Reference to the condition constructor.
         * @param pOntology Reference to the ontology data.
         * @param bVerbose Enables or disables verbose logging.
         */
        constructor(
            pState: TState<S>,
            pTransducer: TTransducer<T>,
            pBranch: TBranch<B>,
            pCondition: TCondition<C>,
            pOntology?: string | O,
            bVerbose?: boolean
        );
        /** Retrieves the constructors. */
        readonly Constructors: IConstructors<S, T, B, C>;
        /** Retrieves the ontology data. */
        readonly Ontology: O | undefined;
        /** Retrieves a reference to the state dispatcher. */
        readonly States: States;
        /** Retrieves a reference to the main state. */
        readonly Main: Main | undefined;
        /** Retrieves a reference to the current instance. */
        readonly Instance: Instance | undefined;
        /**
         * Retrieves the unique ontology hash, which is a unique representation
         * for the ontology.
         */
        readonly Hash: string;
        /** Retrieves if the ontology is loaded successfully. */
        readonly IsLoaded: boolean;
        /** Retrieves if the VFSM is running. */
        readonly IsRunning: boolean;
        /** Retrieves if the VFSM is stopped. */
        readonly IsStopped: boolean;
        /** Retrieves if the VFSM has ended (without pausing or stopping it). */
        readonly IsEnded: boolean;
        /** Retrieves if the VFSM is paused. */
        readonly IsPaused: boolean;
        /** Retrieves the log message list. */
        readonly Log: string[];
        /** Retrieves the total number of states in this ontology. */
        readonly StateCount: number;
        /** Retrieves the total number of transducers in this ontology. */
        readonly TransducerCount: number;
        /** Retrieves the total number of branches in this ontology. */
        readonly BranchCount: number;
        /** Retrieves the total number of conditions in this ontology. */
        readonly ConditionCount: number;
        /** Enables or disables verbose logging. */
        IsVerbose: boolean;
        /** Sets the log function. */
        LogFunc: (sMessage: string) => void;
        /**
         * Logs a message to the log buffer.
         * @param sMessage Specifies the log message.
         */
        LogMessage(sMessage: string): void;
        /**
         * Loads an ontology.
         * @param pOntology Specifies the ontology data.
         * @return Returns `Errors.None` if the ontology is loaded.
         */
        Load(pOntology: O | string): Errors;
        /**
         * Iterates through each item in the ontology.
         * @param fnItem Callback which is invoked for each item.
         * @param sType Specifies the type of items.
         * @return Returns `true` if the iteration was interrupted.
         */
        ForEach<Types extends State | Transducer | Branch | Condition>(
            fnItem: (pItem: Types) => boolean | void,
            sType?:
                | "states"
                | "transducers"
                | "branches"
                | "conditions"
                | "all",
            pExclude?: State | Transducer | Branch | Condition
        ): boolean;
        /** Unloads the ontology. */
        Unload(): void;
        /**
         * Starts a new instance. Throws an error if no ontology is loaded.
         * @param sThreading Specifies if a single or multi threaded instance should be spawned.
         * @param pResume Specifies the optional resume data for the instance.
         * @return Returns a reference to the instance.
         */
        Start(sThreading?: TThreading, pResume?: IInstanceSnapshot): Instance;
        /**
         * Stops all running instances.
         * @return Returns `true` if there was an actual instance running.
         */
        Stop(): boolean;
        /**
         * Pauses the VFSM ontology.
         * @return Returns the snapshot which is necessary to resume the VFSM
         * or `undefined` if no instance was running.
         */
        Pause(): ISnapshot | undefined;
        /**
         * Resumes the VFSM ontology. This will resume all instances that are
         * present in the snapshot data.
         * @param pSnapshot Specifies the snapshot data.
         * @return Returns `true` if the resume is successful and the VFSM is
         * running.
         */
        Resume(pSnapshot: ISnapshot): boolean;
        /**
         * Invoked when an ontology is loaded.
         * @event
         */
        OnLoad(): void;
        /**
         * Invoked when a branch is parsed.
         * @param pBranch Specifies the branch data.
         * @return Returns the branch data or `cancel` if the parsing of the branch
         * should be cancelled.
         * @event
         */
        OnParseBranch(pBranch: IBranch): IBranch | "cancel";
        /**
         * Invoked when a condition is parsed.
         * @param pCondition Specifies the condition data.
         * @return Returns the condition data or `cancel` if the parsing of the
         * condition should be cancelled.
         * @event
         */
        OnParseCondition(pCondition: ICondition): ICondition | "cancel";
        /**
         * Invoked when a state is parsed.
         * @param pState Specifies the state data.
         * @return Returns the state data or `cancel` if the parsing of the state
         * should be cancelled.
         * @event
         */
        OnParseState(pState: IState): IState | "cancel";
        /**
         * Invoked when a transducer is parsed.
         * @param pTransducer Specifies the transducer data.
         * @return Returns the transducer data or `cancel` if the parsing of the
         * transducer should be cancelled.
         * @event
         */
        OnParseTransducer(pTransducer: ITransducer): ITransducer | "cancel";
        /**
         * Invoked when the ontology is about to unload.
         * @event
         */
        OnUnload(): void;
        /**
         * Invoked when the ontology VFSM is started.
         * @event
         */
        OnStart(): void;
        /**
         * Invoked when the ontology VFSM is ended (all stacks are ended).
         * @param bStopped Specifies if the execution was stopped (interrupted).
         * @param pPaused Specifies if the execution was paused.
         * @event
         */
        OnEnd(bStopped: boolean, bPaused: boolean): void;
        /**
         * Invoked when the ontology VFSM is paused.
         * @event
         */
        OnPause(): void;
        /**
         * Invoked when the ontology VFSM is stopped.
         * @event
         */
        OnStop(): void;
        /**
         * Invoked when a stack instance is created.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnInstanceCreate(pInstance: Instance): void;
        /**
         * Invoked when a stack instance is started.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnInstanceStart(pInstance: Instance): void;
        /**
         * Invoked when a stack instance is ended. This event is always invoked
         * when an instance ends normally, is stopped or paused.
         * @param pInstance Reference to the instance.
         * @param sType Specifies why the instance is ended. Can be one of the
         * following values:
         * - `ended`: The instance has ended normally;
         * - `break`: The instance was forced to stop;
         * - `paused`: The instance was paused.
         * @event
         */
        OnInstanceEnd(
            pInstance: Instance,
            sType: "ended" | "stopped" | "paused"
        ): void;
        /**
         * Invoked when a stack instance is paused.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnInstancePause(pInstance: Instance): void;
        /**
         * Invoked when a stack instance is stopped.
         * @param pInstance Reference to the instance.
         * @event
         */
        OnInstanceStop(pInstance: Instance): void;
    }
}

declare module "@tripetto/vfsm/lib/interface" {
    /** Dependencies */
    import { ISerialize } from "@marblecore/serializer";
    import { IBranch } from "@tripetto/vfsm/lib/branches/interface";
    export interface IOntologyProperties extends ISerialize {
        /** Contains the property identifier. */
        Id: string;
    }
    export interface IOntology extends IBranch {}
}

declare module "@tripetto/vfsm/lib/snapshot/snapshot" {
    /** Dependencies */
    import { IInstanceSnapshot } from "@tripetto/vfsm/lib/instance/snapshot";
    export interface ISnapshot {
        /** Contains the instance data. */
        A: IInstanceSnapshot[];
    }
}

declare module "@tripetto/vfsm/lib/store/store" {
    /** Dependencies */
    import { Matrix } from "@marblecore/matrix";
    import { Slot } from "@tripetto/slots";
    import { IData } from "@tripetto/vfsm/lib/store/interface";
    import { StoreItem } from "@tripetto/vfsm/lib/store/item";
    import { TStoreEvents } from "@tripetto/vfsm/lib/store/events";
    export class Store extends Matrix<StoreItem, TStoreEvents> {
        /** Retrieves a collection of slots used in the store. */
        readonly Slots: Slot[];
        /**
         * Retrieves a collection of all verified values. The values are stored
         * per slot. The key of each value object is the hash of the slot. The keys
         * for the contextual values are composed using the condition hashes of the
         * context. If the context is global the key of that value is `*`. If the
         * context array contains a single condition, the value key is the hash of
         * that condition. If the context array contains 2 or more conditions, the
         * hashes of these conditions are used to calculate a new hash. The
         * condition hashes are concatenated in chronological order and then a
         * `SHA2_256` hash is generated for this concatenated string. This hash is
         * used as key.
         */
        readonly Values: IData;
    }
}

declare module "@tripetto/vfsm/lib/store/data" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { StoreItem } from "@tripetto/vfsm/lib/store/item";
    export class Data<T extends TSerializeTypes> {
        /**
         * Creates a new data store.
         * @param pStore Reference to the store.
         * @param pContext Contains the context of the data.
         * @param pData Contains the hot data value.
         * @param sReference Contains the data reference.
         * @param nTime Contains the UTC set time of the data.
         * @param bVerified Specifies if the data is verified.
         */
        constructor(
            pStore: StoreItem<T>,
            pContext: string[],
            pData?: TSerializeTypes,
            sReference?: string,
            nTime?: number,
            bVerified?: boolean
        );
        /** Retrieves the context path of the data. */
        readonly Context: string[];
        /** Retrieves the key for the data. */
        readonly Key: string;
        /** Sets the hot data. */
        Data: TSerializeTypes;
        /** Sets the value. */
        Value: T;
        /** Retrieves the data value as a string. */
        readonly String: string;
        /** Retrieves the data reference. */
        readonly Reference: string;
        /** Sets the verification state of the data. */
        IsVerified: boolean;
        /** Retrieves the data set timestamp. */
        readonly Time: number;
        /**
         * Sets the hot data and optional reference.
         * @param pData Specifies the hot data.
         * @param sReference Specifies a data reference.
         * @return Retrieves the clean value.
         */
        Set(pData: TSerializeTypes, sReference?: string): T;
    }
}

declare module "@tripetto/vfsm/lib/store/interface" {
    /** Dependencies */
    import { TSerializeTypes } from "@marblecore/serializer";
    export interface IDataValue {
        /** Contains the data value. */
        Value: TSerializeTypes;
        /** Contains the data reference. */
        Reference: string;
        /** Contains the UTC set time of the data. */
        Time: number;
        /** Contains the context. */
        Context: string[];
    }
    export interface IDataValues {
        [Key: string]: IDataValue;
    }
    export interface IData {
        [Slot: string]: {
            /** Data slot name. */
            Name: string;
            /** Data slot values. */
            Values: IDataValues;
        };
    }
}

declare module "@tripetto/vfsm/enum/errors" {
    export enum Errors {
        /** No errors were detected. */
        None = 0,
        /** The state data in the ontology is invalid. */
        OntologyInvalidState = 1,
        /** The transducer data in the ontology is invalid. */
        OntologyInvalidTransducer = 2,
        /** The branch data in the ontology is invalid. */
        OntologyInvalidBranch = 4,
        /** The condition data in the ontology is invalid. */
        OntologyInvalidCondition = 8,
        /** The jump reference in the ontology is invalid. */
        OntologyInvalidJump = 16,
        /** The resume data is invalid. */
        ResumeInvalidHash = 32,
        /** One of more resume data slots are invalid. */
        ResumeInvalidData = 64,
        /** The resume stack is invalid. */
        ResumeInvalidStack = 128
    }
}

declare module "@tripetto/vfsm/lib/decorator" {
    import { TState } from "@tripetto/vfsm/lib/states/type";
    import { TTransducer } from "@tripetto/vfsm/lib/transducers/type";
    import { TBranch } from "@tripetto/vfsm/lib/branches/type";
    import { TCondition } from "@tripetto/vfsm/lib/conditions/type";
    export type TTypes = TState | TTransducer | TBranch | TCondition;
    /**
     * Sets the property name for an ontology.
     * @param sProperty Specifies the property name.
     */
    export function ontology<T extends TTypes>(
        sProperty: string
    ): (pType: T) => void;
}

declare module "@tripetto/vfsm/lib/events" {
    /** Dependencies */
    import { IHookPayload, THooks } from "@marblecore/hookup";
    import { Ontology } from "@tripetto/vfsm/lib/ontology";
    import { Instance } from "@tripetto/vfsm/lib/instance/instance";
    import { State } from "@tripetto/vfsm/lib/states/state";
    import { Transducer } from "@tripetto/vfsm/lib/transducers/transducer";
    import { Branch } from "@tripetto/vfsm/lib/branches/branch";
    import { Condition } from "@tripetto/vfsm/lib/conditions/condition";
    export type TOntologyEvents =
        | "OnLoad"
        | "OnUnload"
        | "OnStart"
        | "OnEnd"
        | "OnPause"
        | "OnStop"
        | "OnInstanceCreate"
        | "OnInstanceStart"
        | "OnInstancePause"
        | "OnInstanceStop"
        | "OnInstanceEnd";
    export interface IOntologyEvent<T extends THooks> extends IHookPayload<T> {
        Ontology: Ontology<State, Transducer, Branch, Condition>;
    }
    export interface IOntologyLoadEvent extends IOntologyEvent<"OnLoad"> {}
    export interface IOntologyUnloadEvent extends IOntologyEvent<"OnUnload"> {}
    export interface IOntologyStartEvent extends IOntologyEvent<"OnStart"> {}
    export interface IOntologyPauseEvent extends IOntologyEvent<"OnPause"> {}
    export interface IOntologyStopEvent extends IOntologyEvent<"OnStop"> {}
    export interface IOntologyEndEvent extends IOntologyEvent<"OnEnd"> {
        Stopped: boolean;
        Paused: boolean;
    }
    export interface IOntologyInstanceEvent<T extends THooks>
        extends IOntologyEvent<T> {
        Instance: Instance;
    }
    export interface IOntologyInstanceCreateEvent
        extends IOntologyInstanceEvent<"OnInstanceCreate"> {}
    export interface IOntologyInstanceStartEvent
        extends IOntologyInstanceEvent<"OnInstanceStart"> {}
    export interface IOntologyInstancePauseEvent
        extends IOntologyInstanceEvent<"OnInstancePause"> {}
    export interface IOntologyInstanceStopEvent
        extends IOntologyInstanceEvent<"OnInstanceStop"> {}
    export interface IOntologyInstanceEndEvent
        extends IOntologyInstanceEvent<"OnInstanceEnd"> {
        Type: "ended" | "stopped" | "paused";
    }
}

declare module "@tripetto/vfsm/lib/instance/events" {
    /** Dependencies */
    import { IHookPayload, THooks } from "@marblecore/hookup";
    import { Instance } from "@tripetto/vfsm/lib/instance/instance";
    export type TInstanceEvents =
        | "OnCreate"
        | "OnStart"
        | "OnEnd"
        | "OnPause"
        | "OnStop";
    export interface IInstanceEvent<T extends THooks> extends IHookPayload<T> {
        Instance: Instance;
    }
    export interface IInstanceCreateEvent extends IInstanceEvent<"OnCreate"> {}
    export interface IInstanceStartEvent extends IInstanceEvent<"OnStart"> {}
    export interface IInstancePauseEvent extends IInstanceEvent<"OnPause"> {}
    export interface IInstanceStopEvent extends IInstanceEvent<"OnStop"> {}
    export interface IInstanceEndEvent extends IInstanceEvent<"OnEnd"> {
        Type: "ended" | "stopped" | "paused";
    }
}

declare module "@tripetto/vfsm/lib/branches/events" {
    /** Dependencies */
    import { IHookPayload, THooks } from "@marblecore/hookup";
    import { Instance } from "@tripetto/vfsm/lib/instance/instance";
    import { Branch } from "@tripetto/vfsm/lib/branches/branch";
    export type TBranchEvents =
        | "OnReady"
        | "OnEnter"
        | "OnSkip"
        | "OnLeave"
        | "OnCancel";
    export interface IBranchEvent<T extends THooks> extends IHookPayload<T> {
        Branch: Branch;
        Instance: Instance;
    }
    export interface IBranchReadyEvent extends IBranchEvent<"OnReady"> {}
    export interface IBranchEnterEvent extends IBranchEvent<"OnEnter"> {}
    export interface IBranchSkipEvent extends IBranchEvent<"OnSkip"> {}
    export interface IBranchLeaveEvent extends IBranchEvent<"OnLeave"> {}
    export interface IBranchCancelEvent extends IBranchEvent<"OnCancel"> {}
}

declare module "@tripetto/vfsm/lib/states/events" {
    /** Dependencies */
    import { IHookPayload, THooks } from "@marblecore/hookup";
    import { Instance } from "@tripetto/vfsm/lib/instance/instance";
    import { State } from "@tripetto/vfsm/lib/states/state";
    export type TStateEvents =
        | "OnEnter"
        | "OnSkip"
        | "OnLeave"
        | "OnCancel"
        | "OnValidated";
    export interface IStateEvent<T extends THooks> extends IHookPayload<T> {
        State: State;
        Instance: Instance;
    }
    export interface IStateEnterEvent extends IStateEvent<"OnEnter"> {}
    export interface IStateSkipEvent extends IStateEvent<"OnSkip"> {}
    export interface IStateLeaveEvent extends IStateEvent<"OnLeave"> {}
    export interface IStateCancelEvent extends IStateEvent<"OnCancel"> {}
    export interface IStateValidatedEvent extends IStateEvent<"OnValidated"> {
        Result: "fail" | "pass";
        Type: "initial" | "revalidate";
    }
}

declare module "@tripetto/vfsm/lib/transducers/events" {
    /** Dependencies */
    import { IHookPayload, THooks } from "@marblecore/hookup";
    import { Instance } from "@tripetto/vfsm/lib/instance/instance";
    import { Transducer } from "@tripetto/vfsm/lib/transducers/transducer";
    export type TTransducerEvents =
        | "OnEnter"
        | "OnSkip"
        | "OnLeave"
        | "OnCancel"
        | "OnValidated";
    export interface ITransducerEvent<T extends THooks>
        extends IHookPayload<T> {
        Transducer: Transducer;
        Instance: Instance;
    }
    export interface ITransducerEnterEvent
        extends ITransducerEvent<"OnEnter"> {}
    export interface ITransducerSkipEvent extends ITransducerEvent<"OnSkip"> {}
    export interface ITransducerLeaveEvent
        extends ITransducerEvent<"OnLeave"> {}
    export interface ITransducerCancelEvent
        extends ITransducerEvent<"OnCancel"> {}
    export interface ITransducerValidatedEvent
        extends ITransducerEvent<"OnValidated"> {
        Result: "fail" | "pass";
        Type: "initial" | "revalidate";
        Observed: boolean;
    }
}

declare module "@tripetto/vfsm/lib/store/events" {
    /** Dependencies */
    import { IHookPayload } from "@marblecore/hookup";
    import { Slot } from "@tripetto/slots";
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Data } from "@tripetto/vfsm/lib/store/data";
    import { Store } from "@tripetto/vfsm/lib/store/store";
    export type TStoreEvents = "OnStore";
    export interface IStoreEvent<T extends TSerializeTypes, S extends Slot<T>>
        extends IHookPayload<"OnStore"> {
        Slot: S;
        Data: Data<T>;
        Store: Store;
    }
}

declare module "@tripetto/vfsm/lib/factory" {
    /** Dependencies */
    import { StrictItem, StrictMatrix } from "@marblecore/matrix";
    import { THooks } from "@marblecore/hookup";
    import { State } from "@tripetto/vfsm/lib/states/state";
    import { Branch } from "@tripetto/vfsm/lib/branches/branch";
    import { IOntologyProperties } from "@tripetto/vfsm/lib/interface";
    export interface IFactory<
        Dispatcher extends StrictMatrix<
            Dispatcher,
            Factory<Dispatcher, Ontology, Hooks>,
            void,
            Ontology
        >,
        T extends Factory<Dispatcher, Ontology, Hooks>,
        Ontology extends IOntologyProperties,
        Hooks extends THooks = void
    > {
        readonly Property: string;
        new (pDispatcher: Dispatcher, pOntology: Ontology): T;
    }
    export abstract class Factory<
        Dispatcher extends StrictMatrix<
            Dispatcher,
            Factory<Dispatcher, Ontology, Hooks>,
            void,
            Ontology
        >,
        Ontology extends IOntologyProperties,
        Hooks extends THooks = void
    > extends StrictItem<Dispatcher, Hooks, Ontology> {
        /** Contains the ontology property. */
        static readonly Property: string;
        /**
         * Constructs a new item.
         * @param pDispatcher Reference to the dispatcher.
         * @param sHash Specifies the unique hash for the item.
         */
        constructor(pDispatcher: Dispatcher, sHash: string);
        /** Sets the ancestor state. */
        Ancestor: State | undefined;
        /** Sets the parent branch. */
        Branch: Branch | undefined;
        /** Sets the successor state. */
        Successor: State | undefined;
        /**
         * Maps the ancestor successor to the successor.
         * @return Returns the successor state reference.
         */
        MapSuccessor(): State | undefined;
    }
}

declare module "@tripetto/vfsm/lib/branches/culling" {
    export type TBranchCulling = "first" | "each";
}

declare module "@tripetto/vfsm/lib/states/main" {
    import { State } from "@tripetto/vfsm/lib/states/state";
    import { States } from "@tripetto/vfsm/lib/states/states";
    export class Main extends State {
        /**
         * Constructs a new main state.
         * @param pStates Reference to the state matrix.
         */
        constructor(pStates: States);
        /** Enter condition for the main state. */
        OnEnterCondition(): boolean;
        /** Main state is entered. */
        OnEnter(): void;
        /** Leave condition for the main state. */
        OnLeaveCondition(): boolean;
        /** Main state is validated. */
        OnValidate(): boolean;
    }
}

declare module "@tripetto/vfsm/lib/store/item" {
    import { Slot } from "@tripetto/slots";
    import { Item } from "@marblecore/matrix";
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Stack } from "@tripetto/vfsm/lib/stack/stack";
    import { Store } from "@tripetto/vfsm/lib/store/store";
    import { IStoreSnapshot } from "@tripetto/vfsm/lib/store/snapshot";
    import { Data } from "@tripetto/vfsm/lib/store/data";
    import { TStoreEvents } from "@tripetto/vfsm/lib/store/events";
    import { IDataValues } from "@tripetto/vfsm/lib/store/interface";
    export class StoreItem<
        T extends TSerializeTypes = TSerializeTypes,
        S extends Slot<T> = Slot<T>
    > extends Item<TStoreEvents> {
        /**
         * Constructs a new data store.
         * @param pSlot Reference to the slot.
         * @param pSnapshot Reference to a snapshot.
         */
        constructor(pSlot: S);
        /** Retrieves a reference to the data collection. */
        readonly Store: Store;
        /** Retrieves a reference to the slot. */
        readonly Slot: S;
        /** Retrieves the verified data values. */
        readonly Values: IDataValues;
        /**
         * Retrieves the most relevant data for the supplied stack.
         * @param pStack Reference to the stack.
         * @return Returns the data.
         */
        DataForStack(pStack: Stack | undefined): Data<T>;
        /**
         * Emits a store event.
         * @param pData Reference to the stored data.
         */
        Emit(pData: Data<T>): void;
        /**
         * Snapshots the store.
         * @return Returns the store snapshot.
         */
        Snapshot(): IStoreSnapshot;
        /**
         * Restores the store.
         * @param pSnapshot Specifies the snapshot to restore.
         */
        Restore(pSnapshot: IStoreSnapshot): void;
    }
}

declare module "@tripetto/vfsm/lib/stack/stack" {
    import { Item, Matrix } from "@marblecore/matrix";
    import { Instance } from "@tripetto/vfsm/lib/instance/instance";
    import { Validators } from "@tripetto/vfsm/lib/validators/validators";
    import { Condition } from "@tripetto/vfsm/lib/conditions/condition";
    import { Branch } from "@tripetto/vfsm/lib/branches/branch";
    import { State } from "@tripetto/vfsm/lib/states/state";
    import { IStackSnapshot } from "@tripetto/vfsm/lib/stack/snapshot";
    import { Opcode } from "@tripetto/vfsm/enum/opcodes";
    export class Stack extends Item {
        /**
         * Creates a new stack item.
         * @param pStack Reference to the stack matrix.
         * @param pInstance Reference to the instance.
         * @param pAncestor Reference to the stack item ancestor.
         * @param pState Reference to a state.
         * @param pBranch Reference to a branch.
         * @param pCondition Reference to a condition.
         * @param pBranched Contains a reference to the active branch.
         * @param pTransducers Reference to the transducer validators.
         * @param nOpcode Specifies the opcode.
         * @param nCounter Specifies the number of loops.
         * @param bReturn Specifies if there is a return value.
         * @param bReturnOverride Specifies if a return override value is present.
         * @param pReturn Specifies the return state.
         * @param bBranching Specifies if the stack is branching.
         */
        static Create(
            pStack: Matrix<Stack>,
            pInstance: Instance,
            pAncestor: Stack | undefined,
            pState: State | undefined,
            pBranch?: Branch | undefined,
            pCondition?: Condition | undefined,
            pBranched?: Branch | undefined,
            pValidators?: Validators | undefined,
            nOpcode?: Opcode,
            nCounter?: number,
            bReturn?: boolean,
            bReturnOverride?: boolean,
            pReturn?: State | undefined,
            bBranching?: boolean
        ): Stack;
        /** Retrieves the stack matrix. */
        readonly Parent: Matrix<Stack>;
        /** Contains a reference to the instance. */
        readonly Instance: Instance;
        /** Contains a reference to the ancestor stack item. */
        readonly Ancestor: Stack | undefined;
        /** Reference to the state for the stack item. */
        readonly State: State | undefined;
        /** Reference to the branch for the stack item. */
        readonly Branch: Branch | undefined;
        /** Reference to the condition for the stack item. */
        readonly Condition: Condition | undefined;
        /** Retrieves the stack context. */
        readonly Context: Condition | undefined;
        /** Retrieves if the stack is branching. */
        readonly IsBranch: boolean;
        /** Specifies if the stack item is scheduled for the pursue step. */
        readonly IsScheduled: boolean;
        /** Retrieves if the stack branch will end. */
        readonly IsAtEnd: boolean;
        /**
         * Performs a step.
         * @param nOpcode Specifies the opcode to be executed.
         */
        Step(nOpcode?: Opcode): void;
        /**
         * Verifies if the callback is incremented to the stack.
         * @param bAsynchronous Specifies if the callback was asynchronous. This
         * will reset the stack iteration counter.
         * @return Returns `true` if the callback should be executed.
         */
        Increment(bAsynchronous: boolean): boolean;
        /**
         * Validates the current stack state.
         * @param bSynchronous Specifies the validation is executed in synchronous mode.
         */
        Validate(bSynchronous?: boolean): void;
        /**
         * Destroys the stack item.
         * @return Returns `true` if the function succeeded.
         */
        Destroy(): void;
        /**
         * Snapshots the stack.
         * @return Returns the stack snapshot.
         */
        Snapshot(): IStackSnapshot;
    }
}

declare module "@tripetto/vfsm/lib/instance/snapshot" {
    /** Dependencies */
    import { ITraceSnapshot } from "@tripetto/vfsm/lib/snapshot/trace";
    import { IStoreSnapshot } from "@tripetto/vfsm/lib/store/snapshot";
    export interface IInstanceSnapshot {
        /** Contains the ontology hash. */
        A: string;
        /** Contains the stack trace. */
        B: ITraceSnapshot[];
        /** Contains the slot data. */
        C: IStoreSnapshot[];
        /** Contains the flag data. */
        D: number;
    }
}

declare module "@tripetto/vfsm/lib/threading" {
    export type TThreading = "single" | "multi";
}

declare module "@tripetto/vfsm/lib/branches/type" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { IFactory } from "@tripetto/vfsm/lib/factory";
    import { Branch } from "@tripetto/vfsm/lib/branches/branch";
    import { Branches } from "@tripetto/vfsm/lib/branches/branches";
    import { IBranch } from "@tripetto/vfsm/lib/branches/interface";
    import { TBranchEvents } from "@tripetto/vfsm/lib/branches/events";
    export type TBranch<
        T extends Branch = Branch,
        Ontology extends IBranch = F.TAny
    > = IFactory<Branches, T, Ontology, TBranchEvents>;
}

declare module "@tripetto/vfsm/lib/conditions/type" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { IFactory } from "@tripetto/vfsm/lib/factory";
    import { Condition } from "@tripetto/vfsm/lib/conditions/condition";
    import { Conditions } from "@tripetto/vfsm/lib/conditions/conditions";
    import { ICondition } from "@tripetto/vfsm/lib/conditions/interface";
    export type TCondition<
        T extends Condition = Condition,
        Ontology extends ICondition = F.TAny
    > = IFactory<Conditions, T, Ontology>;
}

declare module "@tripetto/vfsm/lib/constructors" {
    /** Dependencies */
    import { State } from "@tripetto/vfsm/lib/states/state";
    import { TState } from "@tripetto/vfsm/lib/states/type";
    import { Transducer } from "@tripetto/vfsm/lib/transducers/transducer";
    import { TTransducer } from "@tripetto/vfsm/lib/transducers/type";
    import { Branch } from "@tripetto/vfsm/lib/branches/branch";
    import { TBranch } from "@tripetto/vfsm/lib/branches/type";
    import { Condition } from "@tripetto/vfsm/lib/conditions/condition";
    import { TCondition } from "@tripetto/vfsm/lib/conditions/type";
    export interface IConstructors<
        S extends State,
        T extends Transducer,
        B extends Branch,
        C extends Condition
    > {
        State: TState<S>;
        Transducer: TTransducer<T>;
        Branch: TBranch<B>;
        Condition: TCondition<C>;
    }
}

declare module "@tripetto/vfsm/lib/states/type" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { IFactory } from "@tripetto/vfsm/lib/factory";
    import { State } from "@tripetto/vfsm/lib/states/state";
    import { States } from "@tripetto/vfsm/lib/states/states";
    import { IState } from "@tripetto/vfsm/lib/states/interface";
    import { TStateEvents } from "@tripetto/vfsm/lib/states/events";
    export type TState<
        T extends State = State,
        Ontology extends IState = F.TAny
    > = IFactory<States, T, Ontology, TStateEvents>;
}

declare module "@tripetto/vfsm/lib/transducers/type" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { IFactory } from "@tripetto/vfsm/lib/factory";
    import { Transducer } from "@tripetto/vfsm/lib/transducers/transducer";
    import { Transducers } from "@tripetto/vfsm/lib/transducers/transducers";
    import { ITransducer } from "@tripetto/vfsm/lib/transducers/interface";
    import { TTransducerEvents } from "@tripetto/vfsm/lib/transducers/events";
    export type TTransducer<
        T extends Transducer = Transducer,
        Ontology extends ITransducer = F.TAny
    > = IFactory<Transducers, T, Ontology, TTransducerEvents>;
}

declare module "@tripetto/vfsm/lib/store/snapshot" {
    /** Dependencies */
    import { TSerializeTypes } from "@marblecore/serializer";
    export interface IDataSnapshot {
        /** Contains the context hash. */
        A: string;
        /** Contains the data value. */
        B: TSerializeTypes;
        /** Contains the data reference. */
        C: string;
        /** Specifies if the data is verified. */
        D: boolean;
        /** Contains the UTC set time of the data. */
        E: number;
        /** Specifies the context of the data. */
        F: string[];
    }
    export interface IStoreSnapshot {
        /** Contains the store hash. */
        A: string;
        /** Contains the data. */
        B: IDataSnapshot[];
    }
}

declare module "@tripetto/vfsm/lib/validators/validators" {
    /** Dependencies */
    import { Matrix } from "@marblecore/matrix";
    import { Validator } from "@tripetto/vfsm/lib/validators/validator";
    import { Transducer } from "@tripetto/vfsm/lib/transducers/transducer";
    export class Validators extends Matrix<Validator> {
        /**
         * Creates a new transducer validator.
         * @param pTransducer Reference to the transducer.
         * @param bEnabled Specifies if the transducer is enabled.
         * @param bCallback Specifies if there has been a callback.
         * @param bValidated Specifies if the transducer is validated.
         * @param bPass Specifies if the validation is passed.
         */
        Create(
            pTransducer: Transducer,
            bEnabled?: boolean,
            bCallback?: boolean,
            bValidated?: boolean,
            bPass?: boolean
        ): Validator;
    }
}

declare module "@tripetto/vfsm/lib/stack/snapshot" {
    /** Dependencies */
    import { ITransducerSnapshot } from "@tripetto/vfsm/lib/transducers/snapshot";
    import { Opcode } from "@tripetto/vfsm/enum/opcodes";
    export interface IStackSnapshot {
        /** Contains the stack hash. */
        A: string;
        /** Contains the ancestor hash. */
        B: string;
        /** Contains the state hash. */
        C: string;
        /** Contains the branch hash. */
        D: string;
        /** Contains the hash of the branched branch. */
        E: string;
        /** Contains the condition hash. */
        F: string;
        /** Contains the return hash. */
        G: string;
        /** Contains the transducer snapshot. */
        H: ITransducerSnapshot[];
        /** Contains the opcode. */
        I: Opcode;
        /** Contains the counter. */
        J: number;
        /** Contains the flags. */
        K: number;
    }
}

declare module "@tripetto/vfsm/enum/opcodes" {
    export const enum Opcode {
        Push = 0,
        Branch = 1,
        /**
         * An item is awaiting further processing after the branch conditions
         * are processed.
         */
        BranchWait = 2,
        Enter = 3,
        EnterCondition = 4,
        EnterConditionWait = 5,
        Wait = 6,
        LeaveCondition = 7,
        LeaveConditionWait = 8,
        Pursue = 9,
        Pop = 10
    }
}

declare module "@tripetto/vfsm/lib/snapshot/trace" {
    /** Dependencies */
    import { IStackSnapshot } from "@tripetto/vfsm/lib/stack/snapshot";
    export interface ITraceSnapshot {
        /** Contains the stack snapshot. */
        A: IStackSnapshot[];
        /** Contains the stack pointer hash. */
        B: string;
    }
}

declare module "@tripetto/vfsm/lib/validators/validator" {
    import { Item } from "@marblecore/matrix";
    import { Transducer } from "@tripetto/vfsm/lib/transducers/transducer";
    import { ITransducerSnapshot } from "@tripetto/vfsm/lib/transducers/snapshot";
    import { Stack } from "@tripetto/vfsm/lib/stack/stack";
    export class Validator extends Item {
        /**
         * Constructs a new transducer validator.
         * @param pTransducer Reference to the transducer.
         * @param bEnabled Specifies if the transducer is enabled.
         * @param bCallback Specifies if there has been a callback.
         * @param bValidated Specifies if the transducer is validated.
         * @param bPass Specifies if the validation is passed.
         */
        constructor(
            pTransducer: Transducer,
            bEnabled: boolean,
            bCallback: boolean,
            bValidated: boolean,
            bPass: boolean
        );
        /** Retrieves a reference to the transducer. */
        readonly Transducer: Transducer;
        /** Specifies if the transducer is enabled. */
        readonly IsEnabled: boolean;
        /** Retrieves if the transducer is observed. */
        readonly IsObserved: boolean;
        /** Retrieves if the transducer is validated. */
        readonly IsValidated: boolean;
        /** Retrieves if the transducer validation is passed. */
        readonly IsPassed: boolean;
        /**
         * Validates the transducer.
         * @param pStack Reference to the stack.
         * @param bObserved Specifies if the transducer is observed.
         */
        Validate(pStack: Stack, bObserved: boolean): void;
        /**
         * Snapshots the stack transducer validator.
         * @return Returns the stack transducer snapshot.
         */
        Snapshot(): ITransducerSnapshot;
    }
}

declare module "@tripetto/vfsm/lib/transducers/snapshot" {
    export interface ITransducerSnapshot {
        /** Contains the transducer hash. */
        A: string;
        /** Contains the flags. */
        B: number;
    }
}
